@@ -65,13 +65,44 @@ gs_app_list_lookup (GsAppList *list, const gchar *unique_id)
 	return g_hash_table_lookup (list->hash_by_id, unique_id);
 }
 
+static gboolean
+gs_app_list_check_for_duplicate (GsAppList *list, GsApp *app)
+{
+	GsApp *app_old;
+	const gchar *id;
+	const gchar *id_old = NULL;
+
+	/* does not exist */
+	id = gs_app_get_unique_id (app);
+	app_old = g_hash_table_lookup (list->hash_by_id, id);
+	if (app_old == NULL) {
+		g_debug ("adding %s as nothing matched hash", id);
+		return TRUE;
+	}
+
+	/* existing app is a wildcard */
+	id_old = gs_app_get_unique_id (app_old);
+	if (gs_app_has_quirk (app_old, AS_APP_QUIRK_MATCH_ANY_PREFIX)) {
+		g_debug ("adding %s as %s is a wildcard", id, id_old);
+		return TRUE;
+	}
+
+	/* do a sanity check */
+	if (!as_utils_unique_id_equal (id, id_old)) {
+		g_debug ("unique-id non-equal %s as %s but hash matched!",
+			 id, id_old);
+		return TRUE;
+	}
+
+	/* already exists */
+	g_debug ("not adding duplicate %s as %s already exists", id, id_old);
+	return FALSE;
+}
+
 static void
 gs_app_list_add_safe (GsAppList *list, GsApp *app)
 {
 	const gchar *id;
-#if !AS_CHECK_VERSION(0,6,2)
-	guint i;
-#endif
 
 	/* if we're lazy-loading the ID then we can't filter for duplicates */
 	id = gs_app_get_unique_id (app);
@@ -81,28 +112,12 @@ gs_app_list_add_safe (GsAppList *list, GsApp *app)
 	}
 
 	/* check for duplicate */
-	if (g_hash_table_lookup (list->hash_by_id, id) != NULL) {
-		g_debug ("not adding duplicate %s", id);
+	if (!gs_app_list_check_for_duplicate (list, app))
 		return;
-	}
-
-#if !AS_CHECK_VERSION(0,6,2)
-	/* check for duplicate using globs (slower) */
-	for (i = 0; i < list->array->len; i++) {
-		GsApp *app_tmp = g_ptr_array_index (list->array, i);
-		if (as_utils_unique_id_equal (gs_app_get_unique_id (app),
-					      gs_app_get_unique_id (app_tmp))) {
-			g_debug ("not adding duplicate %s as %s exists",
-				 gs_app_get_unique_id (app),
-				 gs_app_get_unique_id (app_tmp));
-			return;
-		}
-	}
-#endif
 
 	/* just use the ref */
 	g_ptr_array_add (list->array, g_object_ref (app));
-	g_hash_table_insert (list->hash_by_id, (gpointer) id, (gpointer) app);
+	g_hash_table_insert (list->hash_by_id, g_strdup (id), g_object_ref (app));
 }
 
 /**
@@ -129,6 +144,35 @@ gs_app_list_add (GsAppList *list, GsApp *app)
 	gs_app_list_add_safe (list, app);
 }
 
+/**
+ * gs_app_list_remove:
+ * @list: A #GsAppList
+ * @app: A #GsApp
+ *
+ * Removes an application from the list. If the application does not exist the
+ * request is ignored.
+ *
+ * Since: 3.22
+ **/
+void
+gs_app_list_remove (GsAppList *list, GsApp *app)
+{
+	AsApp *app_tmp;
+	const gchar *unique_id;
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&list->mutex);
+
+	g_return_if_fail (GS_IS_APP_LIST (list));
+	g_return_if_fail (GS_IS_APP (app));
+
+	/* remove, or ignore if not found */
+	unique_id = gs_app_get_unique_id (app);
+	app_tmp = g_hash_table_lookup (list->hash_by_id, unique_id);
+	if (app_tmp == NULL)
+		return;
+	g_ptr_array_remove (list->array, app_tmp);
+	g_hash_table_remove (list->hash_by_id, unique_id);
+}
+
 /**
  * gs_app_list_index:
  * @list: A #GsAppList
@@ -433,12 +477,10 @@ gs_app_list_init (GsAppList *list)
 {
 	g_mutex_init (&list->mutex);
 	list->array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
-#if AS_CHECK_VERSION(0,6,2)
-	list->hash_by_id = g_hash_table_new ((GHashFunc) as_utils_unique_id_hash,
-					     (GEqualFunc) as_utils_unique_id_equal);
-#else
-	list->hash_by_id = g_hash_table_new (g_str_hash, g_str_equal);
-#endif
+	list->hash_by_id = g_hash_table_new_full ((GHashFunc) as_utils_unique_id_hash,
+						  (GEqualFunc) as_utils_unique_id_equal,
+						  g_free,
+						  (GDestroyNotify) g_object_unref);
 }
 
 /**
