@@ -702,7 +702,9 @@ gs_app_set_state_internal (GsApp *app, AsAppState state)
 	case AS_APP_STATE_INSTALLED:
 		/* installed has to go into an action state */
 		if (state == AS_APP_STATE_UNKNOWN ||
-		    state == AS_APP_STATE_REMOVING)
+		    state == AS_APP_STATE_REMOVING ||
+		    state == AS_APP_STATE_UPDATABLE ||
+		    state == AS_APP_STATE_UPDATABLE_LIVE)
 			state_change_ok = TRUE;
 		break;
 	case AS_APP_STATE_QUEUED_FOR_INSTALL:
@@ -2224,8 +2226,8 @@ gs_app_get_management_plugin (GsApp *app)
  * operations on the #GsApp.
  * Typical values include "packagekit" and "flatpak"
  *
- * It is an error to attempt to change the management plugin once
- * it has been previously set.
+ * It is an error to attempt to change the management plugin once it has been
+ * previously set or to try to use this function on a wildcard application.
  *
  * Since: 3.22
  **/
@@ -2234,6 +2236,15 @@ gs_app_set_management_plugin (GsApp *app, const gchar *management_plugin)
 {
 	g_return_if_fail (GS_IS_APP (app));
 
+	/* plugins cannot adopt wildcard packages */
+	if (gs_app_has_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX)) {
+		g_warning ("plugins should not set the management plugin on "
+			   "%s to %s -- create a new GsApp in refine()!",
+			   gs_app_get_unique_id (app),
+			   management_plugin);
+		return;
+	}
+
 	/* same */
 	if (g_strcmp0 (app->management_plugin, management_plugin) == 0)
 		return;
@@ -2742,16 +2753,29 @@ gs_app_set_install_date (GsApp *app, guint64 install_date)
 gboolean
 gs_app_is_installed (GsApp *app)
 {
-	AsAppState state;
-
 	g_return_val_if_fail (GS_IS_APP (app), FALSE);
+	return (app->state == AS_APP_STATE_INSTALLED) ||
+	       (app->state == AS_APP_STATE_UPDATABLE) ||
+	       (app->state == AS_APP_STATE_UPDATABLE_LIVE) ||
+	       (app->state == AS_APP_STATE_REMOVING);
+}
 
-	state = gs_app_get_state (app);
-
-	return (state == AS_APP_STATE_INSTALLED) ||
-	       (state == AS_APP_STATE_UPDATABLE) ||
-	       (state == AS_APP_STATE_UPDATABLE_LIVE) ||
-	       (state == AS_APP_STATE_REMOVING);
+/**
+ * gs_app_is_updatable:
+ * @app: a #GsApp
+ *
+ * Gets whether the app is updatable or not.
+ *
+ * Returns: %TRUE if the app is updatable, %FALSE otherwise.
+ *
+ * Since: 3.22
+ **/
+gboolean
+gs_app_is_updatable (GsApp *app)
+{
+	g_return_val_if_fail (GS_IS_APP (app), FALSE);
+	return (app->state == AS_APP_STATE_UPDATABLE) ||
+	       (app->state == AS_APP_STATE_UPDATABLE_LIVE);
 }
 
 /**
@@ -3435,6 +3459,9 @@ gs_app_class_init (GsAppClass *klass)
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 	g_object_class_install_property (object_class, PROP_SUMMARY, pspec);
 
+	/**
+	 * GsApp:description:
+	 */
 	pspec = g_param_spec_string ("description", NULL, NULL,
 				     NULL,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
