@@ -48,6 +48,7 @@ typedef struct {
 	SoupSession	*soup_session;
 	gulong		 notify_quirk_id;
 	GtkWidget	*button_install;
+	GsPluginAction	 action;
 } GsPageHelper;
 
 static void
@@ -89,7 +90,7 @@ gs_page_install_authenticate_cb (GtkDialog *dialog,
 	}
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 					   helper->app,
-	                                   GS_PLUGIN_LOADER_ACTION_INSTALL,
+	                                   GS_PLUGIN_ACTION_INSTALL,
 					   helper->cancellable,
 					   gs_page_app_installed_cb,
 					   helper);
@@ -116,7 +117,7 @@ gs_page_remove_authenticate_cb (GtkDialog *dialog,
 	}
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 					   helper->app,
-	                                   GS_PLUGIN_LOADER_ACTION_REMOVE,
+	                                   GS_PLUGIN_ACTION_REMOVE,
 					   helper->cancellable,
 					   gs_page_app_removed_cb,
 					   helper);
@@ -171,7 +172,7 @@ gs_page_app_installed_cb (GObject *source,
 		           error->message);
 		gs_app_notify_failed_modal (helper->app,
 		                            gs_shell_get_window (priv->shell),
-		                            GS_PLUGIN_LOADER_ACTION_INSTALL,
+		                            GS_PLUGIN_ACTION_INSTALL,
 		                            error);
 		return;
 	}
@@ -184,13 +185,14 @@ gs_page_app_installed_cb (GObject *source,
 		           last_error->message);
 		gs_app_notify_failed_modal (helper->app,
 					    gs_shell_get_window (priv->shell),
-					    GS_PLUGIN_LOADER_ACTION_INSTALL,
+					    GS_PLUGIN_ACTION_INSTALL,
 					    last_error);
 		return;
 	}
 
 	/* only show this if the window is not active */
 	if (gs_app_is_installed (helper->app) &&
+	    helper->action == GS_PLUGIN_ACTION_INSTALL &&
 	    !gs_shell_is_active (priv->shell))
 		gs_app_notify_installed (helper->app);
 
@@ -245,7 +247,7 @@ gs_page_app_removed_cb (GObject *source,
 		g_warning ("failed to remove: %s", error->message);
 		gs_app_notify_failed_modal (helper->app,
 		                            gs_shell_get_window (priv->shell),
-		                            GS_PLUGIN_LOADER_ACTION_REMOVE,
+		                            GS_PLUGIN_ACTION_REMOVE,
 		                            error);
 		return;
 	}
@@ -258,7 +260,7 @@ gs_page_app_removed_cb (GObject *source,
 		           last_error->message);
 		gs_app_notify_failed_modal (helper->app,
 					    gs_shell_get_window (priv->shell),
-					    GS_PLUGIN_LOADER_ACTION_REMOVE,
+					    GS_PLUGIN_ACTION_REMOVE,
 					    last_error);
 		return;
 	}
@@ -314,12 +316,13 @@ gs_page_install_app (GsPage *page, GsApp *app, GCancellable *cancellable)
 	}
 
 	helper = g_slice_new0 (GsPageHelper);
+	helper->action = GS_PLUGIN_ACTION_INSTALL;
 	helper->app = g_object_ref (app);
 	helper->page = g_object_ref (page);
 	helper->cancellable = g_object_ref (cancellable);
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 	                                   app,
-	                                   GS_PLUGIN_LOADER_ACTION_INSTALL,
+	                                   helper->action,
 	                                   helper->cancellable,
 	                                   gs_page_app_installed_cb,
 	                                   helper);
@@ -343,7 +346,7 @@ gs_page_update_app_response_cb (GtkDialog *dialog,
 	g_debug ("update %s", gs_app_get_id (helper->app));
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 					   helper->app,
-					   GS_PLUGIN_LOADER_ACTION_UPDATE,
+					   GS_PLUGIN_ACTION_UPDATE,
 					   helper->cancellable,
 					   gs_page_app_installed_cb,
 					   helper);
@@ -417,6 +420,7 @@ gs_page_update_app (GsPage *page, GsApp *app, GCancellable *cancellable)
 
 	/* non-firmware applications do not have to be prepared */
 	helper = g_slice_new0 (GsPageHelper);
+	helper->action = GS_PLUGIN_ACTION_UPDATE;
 	helper->app = g_object_ref (app);
 	helper->page = g_object_ref (page);
 	helper->cancellable = g_object_ref (cancellable);
@@ -437,7 +441,7 @@ gs_page_update_app (GsPage *page, GsApp *app, GCancellable *cancellable)
 	/* generic fallback */
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 					   helper->app,
-					   GS_PLUGIN_LOADER_ACTION_UPDATE,
+					   helper->action,
 					   helper->cancellable,
 					   gs_page_app_installed_cb,
 					   helper);
@@ -461,7 +465,7 @@ gs_page_remove_app_response_cb (GtkDialog *dialog,
 	g_debug ("remove %s", gs_app_get_id (helper->app));
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 					   helper->app,
-					   GS_PLUGIN_LOADER_ACTION_REMOVE,
+					   helper->action,
 					   helper->cancellable,
 					   gs_page_app_removed_cb,
 					   helper);
@@ -478,6 +482,7 @@ gs_page_remove_app (GsPage *page, GsApp *app, GCancellable *cancellable)
 
 	/* pending install */
 	helper = g_slice_new0 (GsPageHelper);
+	helper->action = GS_PLUGIN_ACTION_REMOVE;
 	helper->app = g_object_ref (app);
 	helper->page = g_object_ref (page);
 	helper->cancellable = g_object_ref (cancellable);
@@ -485,7 +490,7 @@ gs_page_remove_app (GsPage *page, GsApp *app, GCancellable *cancellable)
 		g_debug ("remove %s", gs_app_get_id (app));
 		gs_plugin_loader_app_action_async (priv->plugin_loader,
 		                                   app,
-		                                   GS_PLUGIN_LOADER_ACTION_REMOVE,
+		                                   GS_PLUGIN_ACTION_REMOVE,
 		                                   helper->cancellable,
 		                                   gs_page_app_removed_cb,
 		                                   helper);
@@ -555,7 +560,7 @@ gs_page_launch_app (GsPage *page, GsApp *app, GCancellable *cancellable)
 	GsPagePrivate *priv = gs_page_get_instance_private (page);
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 	                                   app,
-	                                   GS_PLUGIN_LOADER_ACTION_LAUNCH,
+	                                   GS_PLUGIN_ACTION_LAUNCH,
 	                                   cancellable,
 	                                   gs_page_app_launched_cb,
 	                                   NULL);
@@ -580,7 +585,7 @@ gs_page_shortcut_add (GsPage *page, GsApp *app, GCancellable *cancellable)
 	GsPagePrivate *priv = gs_page_get_instance_private (page);
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 	                                   app,
-	                                   GS_PLUGIN_LOADER_ACTION_ADD_SHORTCUT,
+	                                   GS_PLUGIN_ACTION_ADD_SHORTCUT,
 	                                   cancellable,
 	                                   gs_page_app_shortcut_added_cb,
 	                                   NULL);
@@ -605,7 +610,7 @@ gs_page_shortcut_remove (GsPage *page, GsApp *app, GCancellable *cancellable)
 	GsPagePrivate *priv = gs_page_get_instance_private (page);
 	gs_plugin_loader_app_action_async (priv->plugin_loader,
 	                                   app,
-	                                   GS_PLUGIN_LOADER_ACTION_REMOVE_SHORTCUT,
+	                                   GS_PLUGIN_ACTION_REMOVE_SHORTCUT,
 	                                   cancellable,
 	                                   gs_page_app_shortcut_removed_cb,
 	                                   NULL);
