@@ -139,6 +139,11 @@ gs_plugin_func (void)
 	GsAppList *list_dup;
 	GsAppList *list_remove;
 	GsApp *app;
+	guint i;
+
+	/* check enums converted */
+	for (i = 0; i < GS_PLUGIN_ACTION_LAST; i++)
+		g_assert (gs_plugin_action_to_string (i) != NULL);
 
 	/* add a couple of duplicate IDs */
 	app = gs_app_new ("a");
@@ -244,6 +249,30 @@ gs_plugin_func (void)
 	g_assert (gs_app_list_lookup (list, "*/b/c/d/e/f") != NULL);
 	g_assert (gs_app_list_lookup (list, "x/x/x/x/x/x") == NULL);
 	g_object_unref (list);
+
+	/* allow duplicating a wildcard */
+	list = gs_app_list_new ();
+	app = gs_app_new ("gimp.desktop");
+	gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	app = gs_app_new ("gimp.desktop");
+	gs_app_set_unique_id (app, "system/flatpak/*/*/gimp.desktop/stable");
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	g_assert_cmpint (gs_app_list_length (list), ==, 2);
+	g_object_unref (list);
+
+	/* remove apps from the list */
+	list = gs_app_list_new ();
+	app = gs_app_new ("a");
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	app = gs_app_new ("a");
+	gs_app_list_remove (list, app);
+	g_object_unref (app);
+	g_assert_cmpint (gs_app_list_length (list), ==, 0);
+	g_object_unref (list);
 }
 
 static void
@@ -331,7 +360,7 @@ gs_plugin_loader_install_func (GsPluginLoader *plugin_loader)
 	gs_app_set_management_plugin (app, "dummy");
 	gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
 	ret = gs_plugin_loader_app_action (plugin_loader, app,
-					   GS_PLUGIN_LOADER_ACTION_INSTALL,
+					   GS_PLUGIN_ACTION_INSTALL,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -341,7 +370,7 @@ gs_plugin_loader_install_func (GsPluginLoader *plugin_loader)
 	/* remove -- we're really testing for return code UNKNOWN,
 	 * but dummy::refine() sets it */
 	ret = gs_plugin_loader_app_action (plugin_loader, app,
-					   GS_PLUGIN_LOADER_ACTION_REMOVE,
+					   GS_PLUGIN_ACTION_REMOVE,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -366,7 +395,7 @@ gs_plugin_loader_error_func (GsPluginLoader *plugin_loader)
 	gs_app_set_management_plugin (app, "dummy");
 	gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
 	ret = gs_plugin_loader_app_action (plugin_loader, app,
-					   GS_PLUGIN_LOADER_ACTION_UPDATE,
+					   GS_PLUGIN_ACTION_UPDATE,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -501,7 +530,7 @@ gs_plugin_loader_distro_upgrades_func (GsPluginLoader *plugin_loader)
 	/* download the update */
 	ret = gs_plugin_loader_app_action (plugin_loader,
 					   app,
-					   GS_PLUGIN_LOADER_ACTION_UPGRADE_DOWNLOAD,
+					   GS_PLUGIN_ACTION_UPGRADE_DOWNLOAD,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -511,7 +540,7 @@ gs_plugin_loader_distro_upgrades_func (GsPluginLoader *plugin_loader)
 	/* trigger the update */
 	ret = gs_plugin_loader_app_action (plugin_loader,
 					   app,
-					   GS_PLUGIN_LOADER_ACTION_UPGRADE_TRIGGER,
+					   GS_PLUGIN_ACTION_UPGRADE_TRIGGER,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -834,7 +863,7 @@ gs_plugin_loader_flatpak_repo_func (GsPluginLoader *plugin_loader)
 
 	/* now install the remote */
 	ret = gs_plugin_loader_app_action (plugin_loader, app,
-					   GS_PLUGIN_LOADER_ACTION_INSTALL,
+					   GS_PLUGIN_ACTION_INSTALL,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -856,7 +885,7 @@ gs_plugin_loader_flatpak_repo_func (GsPluginLoader *plugin_loader)
 	/* check the URL was unmangled */
 	remote_url = g_key_file_get_string (kf, group_name, "url", &error);
 	g_assert_no_error (error);
-	g_assert_cmpstr (remote_url, ==, "http://foo.bar");
+	g_assert_cmpstr (remote_url, ==, "http://foo.bar/apps");
 
 	/* try again, check state is correct */
 	app2 = gs_plugin_loader_file_to_app (plugin_loader,
@@ -870,7 +899,7 @@ gs_plugin_loader_flatpak_repo_func (GsPluginLoader *plugin_loader)
 
 	/* remove it */
 	ret = gs_plugin_loader_app_action (plugin_loader, app,
-					   GS_PLUGIN_LOADER_ACTION_REMOVE,
+					   GS_PLUGIN_ACTION_REMOVE,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -938,7 +967,7 @@ gs_plugin_loader_flatpak_func (GsPluginLoader *plugin_loader)
 	gs_app_set_state (app_source, AS_APP_STATE_AVAILABLE);
 	gs_app_set_url (app_source, AS_URL_KIND_HOMEPAGE, testdir_repourl);
 	ret = gs_plugin_loader_app_action (plugin_loader, app_source,
-					   GS_PLUGIN_LOADER_ACTION_INSTALL,
+					   GS_PLUGIN_ACTION_INSTALL,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -999,7 +1028,7 @@ gs_plugin_loader_flatpak_func (GsPluginLoader *plugin_loader)
 
 	/* install, also installing runtime */
 	ret = gs_plugin_loader_app_action (plugin_loader, app,
-					   GS_PLUGIN_LOADER_ACTION_INSTALL,
+					   GS_PLUGIN_ACTION_INSTALL,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -1046,7 +1075,7 @@ gs_plugin_loader_flatpak_func (GsPluginLoader *plugin_loader)
 
 	/* remove the application */
 	ret = gs_plugin_loader_app_action (plugin_loader, app,
-					   GS_PLUGIN_LOADER_ACTION_REMOVE,
+					   GS_PLUGIN_ACTION_REMOVE,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -1106,7 +1135,7 @@ gs_plugin_loader_authentication_func (GsPluginLoader *plugin_loader)
 
 	/* do an action that returns a URL */
 	ret = gs_plugin_loader_auth_action (plugin_loader, auth,
-					    GS_AUTH_ACTION_REGISTER,
+					    GS_PLUGIN_ACTION_AUTH_REGISTER,
 					    NULL, &error);
 	g_assert_error (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_AUTH_INVALID);
 	g_assert (!ret);
@@ -1117,7 +1146,7 @@ gs_plugin_loader_authentication_func (GsPluginLoader *plugin_loader)
 	app = gs_app_new (NULL);
 	review = as_review_new ();
 	ret = gs_plugin_loader_review_action (plugin_loader, app, review,
-					      GS_PLUGIN_REVIEW_ACTION_REMOVE,
+					      GS_PLUGIN_ACTION_REVIEW_REMOVE,
 					      NULL, &error);
 	g_assert_error (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_AUTH_REQUIRED);
 	g_assert (!ret);
@@ -1125,7 +1154,7 @@ gs_plugin_loader_authentication_func (GsPluginLoader *plugin_loader)
 
 	/* pretend to auth with no credentials */
 	ret = gs_plugin_loader_auth_action (plugin_loader, auth,
-					    GS_AUTH_ACTION_LOGIN,
+					    GS_PLUGIN_ACTION_AUTH_LOGIN,
 					    NULL, &error);
 	g_assert_error (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_AUTH_INVALID);
 	g_assert (!ret);
@@ -1136,7 +1165,7 @@ gs_plugin_loader_authentication_func (GsPluginLoader *plugin_loader)
 	gs_auth_set_username (auth, "dummy");
 	gs_auth_set_password (auth, "dummy");
 	ret = gs_plugin_loader_auth_action (plugin_loader, auth,
-					    GS_AUTH_ACTION_LOGIN,
+					    GS_PLUGIN_ACTION_AUTH_LOGIN,
 					    NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
@@ -1145,7 +1174,7 @@ gs_plugin_loader_authentication_func (GsPluginLoader *plugin_loader)
 	/* do the action that requires a login */
 	review2 = as_review_new ();
 	ret = gs_plugin_loader_review_action (plugin_loader, app, review2,
-					      GS_PLUGIN_REVIEW_ACTION_REMOVE,
+					      GS_PLUGIN_ACTION_REVIEW_REMOVE,
 					      NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
