@@ -53,7 +53,10 @@ typedef struct
 	gchar			*category_of_day;
 	GtkWidget		*search_button;
 	GHashTable		*category_hash;		/* id : GsCategory */
+	GSettings		*settings;
 
+	GtkWidget		*infobar_proprietary;
+	GtkWidget		*label_proprietary;
 	GtkWidget		*bin_featured;
 	GtkWidget		*box_overview;
 	GtkWidget		*box_popular;
@@ -604,6 +607,128 @@ gs_shell_overview_categories_expander_cb (GtkButton *button, GsShellOverview *se
 	gtk_revealer_set_reveal_child (GTK_REVEALER (priv->categories_more), TRUE);
 }
 
+static void
+g_shell_overview_get_sources_cb (GsPluginLoader *plugin_loader,
+				 GAsyncResult *res,
+				 GsShellOverview *self)
+{
+	guint i;
+	g_auto(GStrv) nonfree_ids = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GsAppList) list = NULL;
+	GsShellOverviewPrivate *priv = gs_shell_overview_get_instance_private (self);
+
+	/* get the results */
+	list = gs_plugin_loader_get_sources_finish (plugin_loader, res, &error);
+	if (list == NULL) {
+		if (g_error_matches (error,
+				     G_IO_ERROR,
+				     G_IO_ERROR_CANCELLED)) {
+			g_debug ("get sources cancelled");
+		} else {
+			g_warning ("failed to get sources: %s", error->message);
+		}
+		return;
+	}
+
+	/* enable each */
+	nonfree_ids = g_settings_get_strv (priv->settings, "nonfree-sources");
+	for (i = 0; nonfree_ids[i] != NULL; i++) {
+		GsApp *app;
+		g_autofree gchar *unique_id = NULL;
+
+		/* match the ID from GSettings to an actual GsApp */
+		unique_id = gs_utils_build_unique_id_kind (AS_APP_KIND_SOURCE,
+							   nonfree_ids[i]);
+		app = gs_app_list_lookup (list, unique_id);
+		if (app == NULL) {
+			g_warning ("no source for %s", unique_id);
+			continue;
+		}
+
+		/* depending on the new policy, add or remove the source */
+		if (g_settings_get_boolean (priv->settings, "show-nonfree-software")) {
+			if (gs_app_get_state (app) == AS_APP_STATE_AVAILABLE) {
+				gs_page_install_app (GS_PAGE (self), app,
+						     priv->cancellable);
+			}
+		} else {
+			if (gs_app_get_state (app) == AS_APP_STATE_INSTALLED) {
+				gs_page_remove_app (GS_PAGE (self), app,
+						    priv->cancellable);
+			}
+		}
+	}
+}
+
+static void
+g_shell_overview_rescan_proprietary_sources (GsShellOverview *self)
+{
+	GsShellOverviewPrivate *priv = gs_shell_overview_get_instance_private (self);
+	gs_plugin_loader_get_sources_async (priv->plugin_loader,
+					    GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION,
+					    priv->cancellable,
+					    (GAsyncReadyCallback) g_shell_overview_get_sources_cb,
+					    self);
+}
+
+static void
+g_shell_overview_proprietary_response_cb (GtkInfoBar *info_bar,
+					  gint response_id,
+					  GsShellOverview *self)
+{
+	GsShellOverviewPrivate *priv = gs_shell_overview_get_instance_private (self);
+	g_settings_set_boolean (priv->settings, "show-nonfree-prompt", FALSE);
+	if (response_id == GTK_RESPONSE_CLOSE) {
+		gtk_widget_hide (priv->infobar_proprietary);
+		return;
+	}
+	if (response_id != GTK_RESPONSE_YES)
+		return;
+	g_settings_set_boolean (priv->settings, "show-nonfree-software", TRUE);
+
+	/* actually call into the plugin loader and do the action */
+	g_shell_overview_rescan_proprietary_sources (self);
+}
+
+static void
+gs_shell_overview_refresh_proprietary (GsShellOverview *self)
+{
+	GsShellOverviewPrivate *priv = gs_shell_overview_get_instance_private (self);
+	g_auto(GStrv) nonfree_ids = NULL;
+
+	/* only show if never prompted and have nonfree repos */
+	nonfree_ids = g_settings_get_strv (priv->settings, "nonfree-sources");
+	if (g_settings_get_boolean (priv->settings, "show-nonfree-prompt") &&
+	    !g_settings_get_boolean (priv->settings, "show-nonfree-software") &&
+	    g_strv_length (nonfree_ids) > 0) {
+		g_autoptr(GString) str = g_string_new (NULL);
+		g_autofree gchar *uri = NULL;
+
+		/* get from GSettings, as some distros want to override this */
+		uri = g_settings_get_string (priv->settings, "nonfree-software-uri");
+
+		/* TRANSLATORS: this is the proprietary info bar */
+		g_string_append (str, _("Provides access to additional software, "
+					"including web browsers and games."));
+		g_string_append (str, " ");
+		/* TRANSLATORS: this is the proprietary info bar */
+		g_string_append (str, _("Proprietary software has restrictions "
+					"on use and access to source code."));
+		if (uri != NULL && uri[0] != '\0') {
+			g_string_append (str, "\n");
+			g_string_append_printf (str, "<a href=\"%s\">%s</a>",
+						/* TRANSLATORS: this is the clickable
+						 * link on the proprietary info bar */
+						uri, _("Find out more…"));
+		}
+		gtk_label_set_markup (GTK_LABEL (priv->label_proprietary), str->str);
+		gtk_widget_set_visible (priv->infobar_proprietary, TRUE);
+	} else {
+		gtk_widget_set_visible (priv->infobar_proprietary, FALSE);
+	}
+}
+
 void
 gs_shell_overview_setup (GsShellOverview *self,
 			 GsShell *shell,
@@ -625,6 +750,14 @@ gs_shell_overview_setup (GsShellOverview *self,
 	priv->category_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
 						     g_free, (GDestroyNotify) g_object_unref);
 
+	/* create info bar if not already dismissed in initial-setup */
+	gs_shell_overview_refresh_proprietary (self);
+	gtk_info_bar_add_button (GTK_INFO_BAR (priv->infobar_proprietary),
+				 /* TRANSLATORS: button to turn on proprietary software sources */
+				 _("Enable"), GTK_RESPONSE_YES);
+	g_signal_connect (priv->infobar_proprietary, "response",
+			  G_CALLBACK (g_shell_overview_proprietary_response_cb), self);
+
 	/* avoid a ref cycle */
 	priv->shell = shell;
 
@@ -656,10 +789,28 @@ gs_shell_overview_setup (GsShellOverview *self,
 	               cancellable);
 }
 
+static void
+settings_changed_cb (GSettings *settings,
+		     const gchar *key,
+		     GsShellOverview *self)
+{
+	if (g_strcmp0 (key, "show-nonfree-software") == 0 ||
+	    g_strcmp0 (key, "show-nonfree-prompt") == 0 ||
+	    g_strcmp0 (key, "nonfree-software-uri") == 0 ||
+	    g_strcmp0 (key, "nonfree-sources") == 0) {
+		gs_shell_overview_refresh_proprietary (self);
+	}
+}
+
 static void
 gs_shell_overview_init (GsShellOverview *self)
 {
+	GsShellOverviewPrivate *priv = gs_shell_overview_get_instance_private (self);
 	gtk_widget_init_template (GTK_WIDGET (self));
+	priv->settings = g_settings_new ("org.gnome.software");
+	g_signal_connect (priv->settings, "changed",
+			  G_CALLBACK (settings_changed_cb),
+			  self);
 }
 
 static void
@@ -671,6 +822,7 @@ gs_shell_overview_dispose (GObject *object)
 	g_clear_object (&priv->builder);
 	g_clear_object (&priv->plugin_loader);
 	g_clear_object (&priv->cancellable);
+	g_clear_object (&priv->settings);
 	g_clear_pointer (&priv->category_of_day, g_free);
 	g_clear_pointer (&priv->category_hash, g_hash_table_unref);
 
@@ -710,6 +862,8 @@ gs_shell_overview_class_init (GsShellOverviewClass *klass)
 
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Software/gs-shell-overview.ui");
 
+	gtk_widget_class_bind_template_child_private (widget_class, GsShellOverview, infobar_proprietary);
+	gtk_widget_class_bind_template_child_private (widget_class, GsShellOverview, label_proprietary);
 	gtk_widget_class_bind_template_child_private (widget_class, GsShellOverview, bin_featured);
 	gtk_widget_class_bind_template_child_private (widget_class, GsShellOverview, box_overview);
 	gtk_widget_class_bind_template_child_private (widget_class, GsShellOverview, box_popular);
