@@ -27,6 +27,22 @@
 
 #define	GS_APPSTREAM_MAX_SCREENSHOTS	5
 
+GsApp *
+gs_appstream_create_app (GsPlugin *plugin, AsApp *item, GError **error)
+{
+	const gchar *unique_id = as_app_get_unique_id (item);
+	GsApp *app = gs_plugin_cache_lookup (plugin, unique_id);
+	if (app == NULL) {
+		app = gs_app_new (as_app_get_id (item));
+		gs_app_set_metadata (app, "GnomeSoftware::Creator",
+				     gs_plugin_get_name (plugin));
+		if (!gs_appstream_refine_app (plugin, app, item, error))
+			return NULL;
+		gs_plugin_cache_add (plugin, unique_id, app);
+	}
+	return app;
+}
+
 static AsIcon *
 gs_appstream_get_icon_by_kind (AsApp *app, AsIconKind icon_kind)
 {
@@ -303,7 +319,7 @@ gs_appstream_create_runtime (GsPlugin *plugin,
 		return g_object_ref (app_cache);
 
 	/* save in the cache */
-	gs_plugin_cache_add (plugin, gs_app_get_unique_id (app), app);
+	gs_plugin_cache_add (plugin, NULL, app);
 	return g_steal_pointer (&app);
 }
 
@@ -714,3 +730,265 @@ gs_appstream_refine_app (GsPlugin *plugin,
 
 	return TRUE;
 }
+
+static gboolean
+gs_appstream_store_search_item (GsPlugin *plugin,
+				AsApp *item,
+				gchar **values,
+				GsAppList *list,
+				GCancellable *cancellable,
+				GError **error)
+{
+	AsApp *item_tmp;
+	GPtrArray *addons;
+	guint i;
+	guint match_value;
+	g_autoptr(GsApp) app = NULL;
+
+	/* match against the app or any of the addons */
+	match_value = as_app_search_matches_all (item, values);
+	addons = as_app_get_addons (item);
+	for (i = 0; i < addons->len; i++) {
+		item_tmp = g_ptr_array_index (addons, i);
+		match_value |= as_app_search_matches_all (item_tmp, values);
+	}
+
+	/* no match */
+	if (match_value == 0)
+		return TRUE;
+
+	/* create app */
+	app = gs_appstream_create_app (plugin, item, error);
+	if (app == NULL)
+		return FALSE;
+	gs_app_set_match_value (app, match_value);
+	gs_app_list_add (list, app);
+	return TRUE;
+}
+
+gboolean
+gs_appstream_store_search (GsPlugin *plugin,
+			   AsStore *store,
+			   gchar **values,
+			   GsAppList *list,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	AsApp *item;
+	GPtrArray *array;
+	gboolean ret = TRUE;
+	guint i;
+	g_autoptr(AsProfileTask) ptask = NULL;
+
+	/* search categories for the search term */
+	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
+					  "appstream::search");
+	g_assert (ptask != NULL);
+	array = as_store_get_apps (store);
+	for (i = 0; i < array->len; i++) {
+		if (g_cancellable_set_error_if_cancelled (cancellable, error))
+			return FALSE;
+
+		item = g_ptr_array_index (array, i);
+		ret = gs_appstream_store_search_item (plugin, item,
+						      values, list,
+						      cancellable, error);
+		if (!ret)
+			return FALSE;
+	}
+	return TRUE;
+}
+
+static gboolean
+_as_app_matches_desktop_group_set (AsApp *app, gchar **desktop_groups)
+{
+	guint i;
+	for (i = 0; desktop_groups[i] != NULL; i++) {
+		if (!as_app_has_category (app, desktop_groups[i]))
+			return FALSE;
+	}
+	return TRUE;
+}
+
+static gboolean
+_as_app_matches_desktop_group (AsApp *app, const gchar *desktop_group)
+{
+	g_auto(GStrv) split = g_strsplit (desktop_group, "::", -1);
+	return _as_app_matches_desktop_group_set (app, split);
+}
+
+static void
+gs_appstream_store_add_categories_for_app (GsCategory *parent, AsApp *app)
+{
+	GPtrArray *children;
+	GPtrArray *desktop_groups;
+	GsCategory *category;
+	guint i, j;
+
+	/* find all the sub-categories */
+	children = gs_category_get_children (parent);
+	for (j = 0; j < children->len; j++) {
+		gboolean matched = FALSE;
+		category = GS_CATEGORY (g_ptr_array_index (children, j));
+
+		/* do any desktop_groups match this application */
+		desktop_groups = gs_category_get_desktop_groups (category);
+		for (i = 0; i < desktop_groups->len; i++) {
+			const gchar *desktop_group = g_ptr_array_index (desktop_groups, i);
+			if (_as_app_matches_desktop_group (app, desktop_group)) {
+				matched = TRUE;
+				break;
+			}
+		}
+		if (matched) {
+			gs_category_increment_size (category);
+			gs_category_increment_size (parent);
+		}
+	}
+}
+
+gboolean
+gs_appstream_store_add_category_apps (GsPlugin *plugin,
+				      AsStore *store,
+				      GsCategory *category,
+				      GsAppList *list,
+				      GCancellable *cancellable,
+				      GError **error)
+{
+	GPtrArray *array;
+	GPtrArray *desktop_groups;
+	guint i;
+	guint j;
+	g_autoptr(AsProfileTask) ptask = NULL;
+
+	/* just look at each app in turn */
+	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
+					  "appstream::add-category-apps");
+	g_assert (ptask != NULL);
+	array = as_store_get_apps (store);
+	desktop_groups = gs_category_get_desktop_groups (category);
+	if (desktop_groups->len == 0) {
+		g_warning ("no desktop_groups for %s", gs_category_get_id (category));
+		return TRUE;
+	}
+	for (j = 0; j < desktop_groups->len; j++) {
+		const gchar *desktop_group = g_ptr_array_index (desktop_groups, j);
+		g_auto(GStrv) split = g_strsplit (desktop_group, "::", -1);
+
+		/* match the app */
+		for (i = 0; i < array->len; i++) {
+			AsApp *item;
+			g_autoptr(GsApp) app = NULL;
+
+			/* no ID is invalid */
+			item = g_ptr_array_index (array, i);
+			if (as_app_get_id (item) == NULL)
+				continue;
+
+			/* match all the desktop groups */
+			if (!_as_app_matches_desktop_group_set (item, split))
+				continue;
+
+			/* add all the data we can */
+			app = gs_appstream_create_app (plugin, item, error);
+			if (app == NULL)
+				return FALSE;
+			gs_app_list_add (list, app);
+		}
+	}
+	return TRUE;
+}
+
+gboolean
+gs_appstream_store_add_categories (GsPlugin *plugin,
+				   AsStore *store,
+				   GPtrArray *list,
+				   GCancellable *cancellable,
+				   GError **error)
+{
+	AsApp *app;
+	GPtrArray *array;
+	guint i;
+	guint j;
+	g_autoptr(AsProfileTask) ptask = NULL;
+
+	/* find out how many packages are in each category */
+	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
+					  "appstream::add-categories");
+	g_assert (ptask != NULL);
+	array = as_store_get_apps (store);
+	for (i = 0; i < array->len; i++) {
+		app = g_ptr_array_index (array, i);
+		if (as_app_get_id (app) == NULL)
+			continue;
+		if (as_app_get_priority (app) < 0)
+			continue;
+		for (j = 0; j < list->len; j++) {
+			GsCategory *parent = GS_CATEGORY (g_ptr_array_index (list, j));
+			gs_appstream_store_add_categories_for_app (parent, app);
+		}
+	}
+	return TRUE;
+}
+
+gboolean
+gs_appstream_add_popular (GsPlugin *plugin,
+			  AsStore *store,
+			  GsAppList *list,
+			  GCancellable *cancellable,
+			  GError **error)
+{
+	AsApp *item;
+	GPtrArray *array;
+	guint i;
+	g_autoptr(AsProfileTask) ptask = NULL;
+
+	/* find out how many packages are in each category */
+	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
+					  "appstream::add-popular");
+	g_assert (ptask != NULL);
+	array = as_store_get_apps (store);
+	for (i = 0; i < array->len; i++) {
+		g_autoptr(GsApp) app = NULL;
+		item = g_ptr_array_index (array, i);
+		if (as_app_get_id (item) == NULL)
+			continue;
+		if (!as_app_has_kudo (item, "GnomeSoftware::popular"))
+			continue;
+		app = gs_app_new (as_app_get_id (item));
+		gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
+		gs_app_list_add (list, app);
+	}
+	return TRUE;
+}
+
+gboolean
+gs_appstream_add_featured (GsPlugin *plugin,
+			   AsStore *store,
+			   GsAppList *list,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	AsApp *item;
+	GPtrArray *array;
+	guint i;
+	g_autoptr(AsProfileTask) ptask = NULL;
+
+	/* find out how many packages are in each category */
+	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
+					  "appstream::add-featured");
+	g_assert (ptask != NULL);
+	array = as_store_get_apps (store);
+	for (i = 0; i < array->len; i++) {
+		g_autoptr(GsApp) app = NULL;
+		item = g_ptr_array_index (array, i);
+		if (as_app_get_id (item) == NULL)
+			continue;
+		if (as_app_get_metadata_item (item, "GnomeSoftware::FeatureTile-css") == NULL)
+			continue;
+		app = gs_app_new (as_app_get_id (item));
+		gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
+		gs_app_list_add (list, app);
+	}
+	return TRUE;
+}
