@@ -42,14 +42,67 @@ struct _GsFlatpak {
 	GFileMonitor		*monitor;
 	AsAppScope		 scope;
 	GsPlugin		*plugin;
+	AsStore			*store;
 };
 
 G_DEFINE_TYPE (GsFlatpak, gs_flatpak, G_TYPE_OBJECT)
 
+/* we have to do this until we hard dep on 0.6.11 */
+#define _FLATPAK_CHECK_VERSION(major,minor,micro)    \
+    (FLATPAK_MAJOR_VERSION > (major) || \
+     (FLATPAK_MAJOR_VERSION == (major) && FLATPAK_MINOR_VERSION > (minor)) || \
+     (FLATPAK_MAJOR_VERSION == (major) && FLATPAK_MINOR_VERSION == (minor) && \
+      FLATPAK_MICRO_VERSION >= (micro)))
+
 static gboolean
 gs_flatpak_refresh_appstream (GsFlatpak *self, guint cache_age,
+			      GsPluginRefreshFlags flags,
 			      GCancellable *cancellable, GError **error);
 
+static gchar *
+gs_flatpak_build_id (FlatpakRef *xref)
+{
+	if (flatpak_ref_get_kind (xref) == FLATPAK_REF_KIND_APP) {
+		return g_strdup_printf ("%s.desktop",
+					flatpak_ref_get_name (xref));
+	}
+	return g_strdup_printf ("%s.runtime",
+				flatpak_ref_get_name (xref));
+}
+
+static GsApp *
+gs_plugin_create_app (GsFlatpak *self, FlatpakRef *xref)
+{
+	GsApp *app_cached;
+	g_autofree gchar *id = NULL;
+	g_autoptr(GsApp) app = NULL;
+
+	/* create a temp GsApp */
+	id = gs_flatpak_build_id (xref);
+	app = gs_app_new (id);
+	gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_FLATPAK);
+	gs_app_set_branch (app, flatpak_ref_get_branch (xref));
+	if (flatpak_installation_get_is_user (self->installation)) {
+		gs_app_set_scope (app, AS_APP_SCOPE_USER);
+	} else {
+		gs_app_set_scope (app, AS_APP_SCOPE_SYSTEM);
+	}
+	if (flatpak_ref_get_kind (xref) == FLATPAK_REF_KIND_APP) {
+		gs_app_set_kind (app, AS_APP_KIND_DESKTOP);
+	} else if (flatpak_ref_get_kind (xref) == FLATPAK_REF_KIND_RUNTIME) {
+		gs_app_set_kind (app, AS_APP_KIND_RUNTIME);
+	}
+
+	/* we already have one, returned the ref'd cached copy */
+	app_cached = gs_plugin_cache_lookup (self->plugin, gs_app_get_unique_id (app));
+	if (app_cached != NULL)
+		return app_cached;
+
+	/* no existing match, just steal the temp object */
+	gs_plugin_cache_add (self->plugin, NULL, app);
+	return g_steal_pointer (&app);
+}
+
 static void
 gs_plugin_flatpak_changed_cb (GFileMonitor *monitor,
 			      GFile *child,
@@ -68,14 +121,227 @@ gs_plugin_flatpak_changed_cb (GFileMonitor *monitor,
 	}
 
 	/* if this is a new remote, get the AppStream data */
-	if (!gs_flatpak_refresh_appstream (self, G_MAXUINT, NULL, &error_md)) {
+	if (!gs_flatpak_refresh_appstream (self, G_MAXUINT, 0, NULL, &error_md)) {
 		g_warning ("failed to get initial available data: %s",
 			   error_md->message);
 	}
+}
 
-	/* ensure the AppStream symlink cache is up to date */
-	if (!gs_flatpak_symlinks_rebuild (self->installation, NULL, &error))
-		g_warning ("failed to check symlinks: %s", error->message);
+static void
+gs_flatpak_remove_prefixed_names (AsApp *app)
+{
+	GHashTable *names;
+	GList *l;
+	g_autoptr(GList) keys = NULL;
+
+	names = as_app_get_names (app);
+	keys = g_hash_table_get_keys (names);
+	for (l = keys; l != NULL; l = l->next) {
+		const gchar *locale = l->data;
+		const gchar *value = g_hash_table_lookup (names, locale);
+		if (value == NULL)
+			continue;
+		if (!g_str_has_prefix (value, "(Nightly) "))
+			continue;
+		as_app_set_name (app, locale, value + 10);
+	}
+}
+
+static gboolean
+gs_flatpak_add_apps_from_xremote (GsFlatpak *self,
+				  FlatpakRemote *xremote,
+				  GCancellable *cancellable,
+				  GError **error)
+{
+	GPtrArray *apps;
+	guint i;
+	g_autofree gchar *appstream_dir_fn = NULL;
+	g_autofree gchar *appstream_fn = NULL;
+	g_autofree gchar *default_branch = NULL;
+	g_autofree gchar *only_app_id = NULL;
+	g_autoptr(AsStore) store = NULL;
+	g_autoptr(GFile) appstream_dir = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GPtrArray) app_filtered = NULL;
+
+	/* get the AppStream data location */
+	appstream_dir = flatpak_remote_get_appstream_dir (xremote, NULL);
+	if (appstream_dir == NULL) {
+		g_debug ("no appstream dir for %s, skipping",
+			 flatpak_remote_get_name (xremote));
+		return TRUE;
+	}
+
+	/* load the file into a temp store */
+	appstream_dir_fn = g_file_get_path (appstream_dir);
+	appstream_fn = g_build_filename (appstream_dir_fn,
+					 "appstream.xml.gz", NULL);
+	if (!g_file_test (appstream_fn, G_FILE_TEST_EXISTS)) {
+		g_debug ("no %s appstream metadata found: %s",
+			 flatpak_remote_get_name (xremote),
+			 appstream_fn);
+		return TRUE;
+	}
+	file = g_file_new_for_path (appstream_fn);
+	store = as_store_new ();
+	as_store_set_add_flags (store,
+				AS_STORE_ADD_FLAG_USE_UNIQUE_ID |
+				AS_STORE_ADD_FLAG_ONLY_NATIVE_LANGS);
+	as_store_set_search_match (store,
+				   AS_APP_SEARCH_MATCH_MIMETYPE |
+				   AS_APP_SEARCH_MATCH_PKGNAME |
+				   AS_APP_SEARCH_MATCH_COMMENT |
+				   AS_APP_SEARCH_MATCH_NAME |
+				   AS_APP_SEARCH_MATCH_KEYWORD |
+				   AS_APP_SEARCH_MATCH_ID);
+	if (!as_store_from_file (store, file, NULL, cancellable, error)) {
+		return FALSE;
+	}
+
+	/* ensure the token cache */
+	as_store_load_search_cache (store);
+
+	/* only add the specific app for noenumerate=true */
+	if (flatpak_remote_get_noenumerate (xremote)) {
+		g_autofree gchar *tmp = NULL;
+		tmp = g_strdup (flatpak_remote_get_name (xremote));
+		g_strdelimit (tmp, "-", '\0');
+		only_app_id = g_strdup_printf ("%s.desktop", tmp);
+	}
+
+	default_branch = flatpak_remote_get_default_branch (xremote);
+
+	/* get all the apps and fix them up */
+	apps = as_store_get_apps (store);
+	app_filtered = g_ptr_array_new ();
+	for (i = 0; i < apps->len; i++) {
+		AsApp *app = g_ptr_array_index (apps, i);
+
+		/* filter to app */
+		if (only_app_id != NULL &&
+		    g_strcmp0 (as_app_get_id (app), only_app_id) != 0) {
+			as_app_set_kind (app, AS_APP_KIND_UNKNOWN);
+			continue;
+		}
+
+		/* filter by branch */
+		if (default_branch != NULL &&
+		    g_strcmp0 (as_app_get_branch (app), default_branch) != 0) {
+			g_debug ("not adding app with branch %s as filtering to %s",
+				 as_app_get_branch (app), default_branch);
+			continue;
+		}
+
+		/* fix the names when using old versions of appstream-compose */
+		gs_flatpak_remove_prefixed_names (app);
+
+		/* add */
+		as_app_set_scope (app, self->scope);
+		as_app_set_origin (app, flatpak_remote_get_name (xremote));
+		as_app_add_keyword (app, NULL, "flatpak");
+		g_debug ("adding %s", as_app_get_unique_id (app));
+		g_ptr_array_add (app_filtered, app);
+	}
+
+	/* add them to the main store */
+	as_store_add_apps (self->store, app_filtered);
+	return TRUE;
+}
+
+static void
+gs_flatpak_rescan_installed (GsFlatpak *self, GCancellable *cancellable, GError **error)
+{
+	GPtrArray *icons;
+	const gchar *fn;
+	guint i;
+	g_autoptr(GFile) path = NULL;
+	g_autoptr(GDir) dir = NULL;
+	g_autofree gchar *path_str = NULL;
+	g_autofree gchar *path_exports = NULL;
+	g_autofree gchar *path_apps = NULL;
+
+	/* add all installed desktop files */
+	path = flatpak_installation_get_path (self->installation);
+	path_str = g_file_get_path (path);
+	path_exports = g_build_filename (path_str, "exports", NULL);
+	path_apps = g_build_filename (path_exports, "share", "applications", NULL);
+	dir = g_dir_open (path_apps, 0, NULL);
+	if (dir == NULL)
+		return;
+	while ((fn = g_dir_read_name (dir)) != NULL) {
+		g_autofree gchar *fn_desktop = NULL;
+		g_autoptr(GError) error_local = NULL;
+		g_autoptr(AsApp) app = NULL;
+
+		/* ignore */
+		if (g_strcmp0 (fn, "mimeinfo.cache") == 0)
+			continue;
+
+		/* parse desktop files */
+		app = as_app_new ();
+		fn_desktop = g_build_filename (path_apps, fn, NULL);
+		if (!as_app_parse_file (app, fn_desktop, 0, &error_local)) {
+			g_warning ("failed to parse %s: %s",
+				   fn_desktop, error_local->message);
+			continue;
+		}
+
+		/* fix up icons */
+		icons = as_app_get_icons (app);
+		for (i = 0; i < icons->len; i++) {
+			AsIcon *ic = g_ptr_array_index (icons, i);
+			if (as_icon_get_kind (ic) == AS_ICON_KIND_UNKNOWN) {
+				as_icon_set_kind (ic, AS_ICON_KIND_STOCK);
+				as_icon_set_prefix (ic, path_exports);
+			}
+		}
+
+		/* fix the names when using old versions of appstream-compose */
+		gs_flatpak_remove_prefixed_names (app);
+
+		/* add */
+		as_app_set_state (app, AS_APP_STATE_INSTALLED);
+		as_app_set_scope (app, self->scope);
+		as_app_set_source_kind (app, AS_APP_SOURCE_KIND_DESKTOP);
+		as_app_set_source_file (app, fn_desktop);
+		as_app_set_icon_path (app, path_exports);
+		as_app_add_keyword (app, NULL, "flatpak");
+		as_store_add_app (self->store, app);
+	}
+}
+
+static gboolean
+gs_flatpak_rescan_appstream_store (GsFlatpak *self,
+				   GCancellable *cancellable,
+				   GError **error)
+{
+	guint i;
+	g_autoptr(GPtrArray) xremotes = NULL;
+
+	/* remove all components */
+	as_store_remove_all (self->store);
+
+	/* go through each remote adding metadata */
+	xremotes = flatpak_installation_list_remotes (self->installation,
+						      cancellable,
+						      error);
+	if (xremotes == NULL) {
+		return FALSE;
+	}
+	for (i = 0; i < xremotes->len; i++) {
+		FlatpakRemote *xremote = g_ptr_array_index (xremotes, i);
+		if (flatpak_remote_get_disabled (xremote))
+			continue;
+		g_debug ("found remote %s",
+			 flatpak_remote_get_name (xremote));
+		if (!gs_flatpak_add_apps_from_xremote (self, xremote, cancellable, error))
+			return FALSE;
+	}
+
+	/* add any installed files without AppStream info */
+	gs_flatpak_rescan_installed (self, cancellable, error);
+
+	return TRUE;
 }
 
 gboolean
@@ -84,6 +350,16 @@ gs_flatpak_setup (GsFlatpak *self, GCancellable *cancellable, GError **error)
 	const gchar *destdir;
 	g_autoptr(AsProfileTask) ptask = NULL;
 
+	/* load just the wildcards */
+	if (!as_store_load (self->store,
+			    AS_STORE_LOAD_FLAG_ONLY_MERGE_APPS |
+			    AS_STORE_LOAD_FLAG_ONLY_UNCOMPRESSED |
+			    AS_STORE_LOAD_FLAG_APP_INFO_USER |
+			    AS_STORE_LOAD_FLAG_APP_INFO_SYSTEM,
+			    cancellable, error)) {
+		return FALSE;
+	}
+
 	/* we use a permissions helper to elevate privs */
 	ptask = as_profile_start_literal (gs_plugin_get_profile (self->plugin),
 					  "flatpak::ensure-origin");
@@ -118,16 +394,34 @@ gs_flatpak_setup (GsFlatpak *self, GCancellable *cancellable, GError **error)
 	g_signal_connect (self->monitor, "changed",
 			  G_CALLBACK (gs_plugin_flatpak_changed_cb), self);
 
-	/* ensure the AppStream symlink cache is up to date */
-	if (!gs_flatpak_symlinks_rebuild (self->installation, cancellable, error))
+	/* ensure the legacy AppStream symlink cache is deleted */
+	if (!gs_flatpak_symlinks_cleanup (self->installation, cancellable, error))
 		return FALSE;
 
 	/* success */
 	return TRUE;
 }
 
+static gboolean
+gs_flatpak_refresh_appstream_remote (GsFlatpak *self,
+				     const gchar *remote_name,
+				     GCancellable *cancellable,
+				     GError **error)
+{
+	if (!flatpak_installation_update_appstream_sync (self->installation,
+							 remote_name,
+							 NULL,
+							 NULL,
+							 cancellable,
+							 error)) {
+		return FALSE;
+	}
+	return TRUE;
+}
+
 static gboolean
 gs_flatpak_refresh_appstream (GsFlatpak *self, guint cache_age,
+			      GsPluginRefreshFlags flags,
 			      GCancellable *cancellable, GError **error)
 {
 	gboolean ret;
@@ -135,7 +429,9 @@ gs_flatpak_refresh_appstream (GsFlatpak *self, guint cache_age,
 	guint i;
 	g_autoptr(GPtrArray) xremotes = NULL;
 
-	xremotes = flatpak_installation_list_remotes (self->installation, cancellable,
+	/* get remotes */
+	xremotes = flatpak_installation_list_remotes (self->installation,
+						      cancellable,
 						      error);
 	if (xremotes == NULL)
 		return FALSE;
@@ -172,16 +468,14 @@ gs_flatpak_refresh_appstream (GsFlatpak *self, guint cache_age,
 		/* download new data */
 		g_debug ("%s is %u seconds old, so downloading new data",
 			 remote_name, tmp);
-		ret = flatpak_installation_update_appstream_sync (self->installation,
-								  remote_name,
-								  NULL, /* arch */
-								  NULL, /* out_changed */
-								  cancellable,
-								  &error_local);
+		ret = gs_flatpak_refresh_appstream_remote (self,
+							   remote_name,
+							   cancellable,
+							   &error_local);
 		if (!ret) {
 			if (g_error_matches (error_local,
-					     G_IO_ERROR,
-					     G_IO_ERROR_FAILED)) {
+					     GS_PLUGIN_ERROR,
+					     GS_PLUGIN_ERROR_FAILED)) {
 				g_debug ("Failed to get AppStream metadata: %s",
 					 error_local->message);
 				/* don't try to fetch this again until refresh() */
@@ -190,6 +484,13 @@ gs_flatpak_refresh_appstream (GsFlatpak *self, guint cache_age,
 						     GUINT_TO_POINTER (1));
 				continue;
 			}
+			if ((flags & GS_PLUGIN_REFRESH_FLAGS_INTERACTIVE) == 0) {
+				g_warning ("Failed to get AppStream metadata: %s [%s:%i]",
+					   error_local->message,
+					   g_quark_to_string (error_local->domain),
+					   error_local->code);
+				continue;
+			}
 			g_set_error (error,
 				     GS_PLUGIN_ERROR,
 				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
@@ -207,11 +508,10 @@ gs_flatpak_refresh_appstream (GsFlatpak *self, guint cache_age,
 		something_changed = TRUE;
 	}
 
-	/* ensure the AppStream symlink cache is up to date */
-	if (something_changed) {
-		if (!gs_flatpak_symlinks_rebuild (self->installation,
-						  cancellable,
-						  error))
+	/* ensure the AppStream store is up to date */
+	if (something_changed ||
+	    as_store_get_size (self->store) == 0) {
+		if (!gs_flatpak_rescan_appstream_store (self, cancellable, error))
 			return FALSE;
 	}
 
@@ -294,44 +594,11 @@ gs_flatpak_set_metadata_installed (GsFlatpak *self, GsApp *app,
 		gs_app_set_size_installed (app, size_installed);
 }
 
-static gchar *
-gs_flatpak_build_id (FlatpakRef *xref)
-{
-	if (flatpak_ref_get_kind (xref) == FLATPAK_REF_KIND_APP) {
-		return g_strdup_printf ("%s.desktop",
-					flatpak_ref_get_name (xref));
-	}
-	return g_strdup_printf ("%s.runtime",
-				flatpak_ref_get_name (xref));
-}
-
-static gchar *
-gs_flatpak_build_unique_id (FlatpakInstallation *installation, FlatpakRef *xref)
-{
-	AsAppKind kind = AS_APP_KIND_DESKTOP;
-	AsAppScope scope = AS_APP_SCOPE_SYSTEM;
-	g_autofree gchar *id = NULL;
-
-	/* use a different prefix if we're somehow running this as per-user */
-	if (flatpak_installation_get_is_user (installation))
-		scope = AS_APP_SCOPE_USER;
-	if (flatpak_ref_get_kind (xref) == FLATPAK_REF_KIND_RUNTIME)
-		kind = AS_APP_KIND_RUNTIME;
-	id = gs_flatpak_build_id (xref);
-	return as_utils_unique_id_build (scope,
-					 AS_BUNDLE_KIND_FLATPAK,
-					 NULL,	/* origin */
-					 kind,
-					 id,
-					 flatpak_ref_get_branch (xref));
-}
-
 static GsApp *
 gs_flatpak_create_installed (GsFlatpak *self,
 			     FlatpakInstalledRef *xref,
 			     GError **error)
 {
-	g_autofree gchar *unique_id = NULL;
 	g_autoptr(AsIcon) icon = NULL;
 	g_autoptr(GsApp) app = NULL;
 
@@ -356,14 +623,7 @@ gs_flatpak_create_installed (GsFlatpak *self,
 	}
 
 	/* create new object */
-	unique_id = gs_flatpak_build_unique_id (self->installation,
-						FLATPAK_REF (xref));
-	app = gs_plugin_cache_lookup (self->plugin, unique_id);
-	if (app == NULL) {
-		g_autofree gchar *id = gs_flatpak_build_id (FLATPAK_REF (xref));
-		app = gs_app_new (id);
-		gs_plugin_cache_add (self->plugin, unique_id, app);
-	}
+	app = gs_plugin_create_app (self, FLATPAK_REF (xref));
 	gs_flatpak_set_metadata_installed (self, app, xref);
 
 	switch (flatpak_ref_get_kind (FLATPAK_REF(xref))) {
@@ -403,7 +663,7 @@ gs_flatpak_add_installed (GsFlatpak *self, GsAppList *list,
 	guint i;
 
 	/* if we've never ever run before, get the AppStream data */
-	if (!gs_flatpak_refresh_appstream (self, G_MAXUINT,
+	if (!gs_flatpak_refresh_appstream (self, G_MAXUINT, 0,
 					   cancellable,
 					   &error_md)) {
 		g_warning ("failed to get initial available data: %s",
@@ -526,6 +786,7 @@ gs_flatpak_app_install_source (GsFlatpak *self, GsApp *app,
 			       GError **error)
 {
 	const gchar *gpg_key;
+	const gchar *branch;
 	g_autoptr(FlatpakRemote) xremote = NULL;
 
 	/* only process this source if was created for this plugin */
@@ -548,8 +809,8 @@ gs_flatpak_app_install_source (GsFlatpak *self, GsApp *app,
 
 	/* create a new remote */
 	xremote = flatpak_remote_new (gs_app_get_id (app));
+	flatpak_remote_set_url (xremote, gs_app_get_metadata_item (app, "flatpak::url"));
 	flatpak_remote_set_noenumerate (xremote, FALSE);
-	flatpak_remote_set_url (xremote, gs_app_get_url (app, AS_URL_KIND_HOMEPAGE));
 	if (gs_app_get_summary (app) != NULL)
 		flatpak_remote_set_title (xremote, gs_app_get_summary (app));
 
@@ -567,6 +828,11 @@ gs_flatpak_app_install_source (GsFlatpak *self, GsApp *app,
 		flatpak_remote_set_gpg_verify (xremote, FALSE);
 	}
 
+	/* default branch */
+	branch = gs_app_get_branch (app);
+	if (branch != NULL)
+		flatpak_remote_set_default_branch (xremote, branch);
+
 	/* install it */
 	gs_app_set_state (app, AS_APP_STATE_INSTALLING);
 	if (!flatpak_installation_modify_remote (self->installation,
@@ -577,6 +843,10 @@ gs_flatpak_app_install_source (GsFlatpak *self, GsApp *app,
 		return FALSE;
 	}
 
+	/* refresh the AppStream data manually */
+	if (!gs_flatpak_add_apps_from_xremote (self, xremote, cancellable, error))
+		return FALSE;
+
 	/* success */
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
 	return TRUE;
@@ -590,13 +860,6 @@ gs_flatpak_add_updates (GsFlatpak *self, GsAppList *list,
 	guint i;
 	g_autoptr(GPtrArray) xrefs = NULL;
 
-	/* manually drop the cache */
-	if (0&&!flatpak_installation_drop_caches (self->installation,
-						  cancellable,
-						  error)) {
-		return FALSE;
-	}
-
 	/* get all the installed apps (no network I/O) */
 	xrefs = flatpak_installation_list_installed_refs (self->installation,
 							  cancellable,
@@ -628,9 +891,8 @@ gs_flatpak_add_updates (GsFlatpak *self, GsAppList *list,
 			g_warning ("failed to add flatpak: %s", error_local->message);
 			continue;
 		}
-		if (gs_app_get_state (app) == AS_APP_STATE_INSTALLED)
-			gs_app_set_state (app, AS_APP_STATE_UNKNOWN);
 		gs_app_set_state (app, AS_APP_STATE_UPDATABLE_LIVE);
+		gs_app_set_size_download (app, 0);
 		gs_app_list_add (list, app);
 	}
 
@@ -660,11 +922,17 @@ gs_flatpak_refresh (GsFlatpak *self,
 	/* give all the repos a second chance */
 	g_hash_table_remove_all (self->broken_remotes);
 
+	/* manually drop the cache */
+	if (!flatpak_installation_drop_caches (self->installation,
+					       cancellable,
+					       error)) {
+		return FALSE;
+	}
+
 	/* update AppStream metadata */
 	if (flags & GS_PLUGIN_REFRESH_FLAGS_METADATA) {
-		if (!gs_flatpak_refresh_appstream (self,
-						   cache_age, cancellable,
-						   error))
+		if (!gs_flatpak_refresh_appstream (self, cache_age, flags,
+						   cancellable, error))
 			return FALSE;
 	}
 
@@ -956,11 +1224,11 @@ gs_plugin_refine_item_origin (GsFlatpak *self,
 static gboolean
 gs_flatpak_app_matches_xref (GsFlatpak *self, GsApp *app, FlatpakRef *xref)
 {
-	g_autofree gchar *id = NULL;
+	g_autoptr(GsApp) app_tmp = gs_plugin_create_app (self, xref);
 
 	/* check ID */
-	id = gs_flatpak_build_unique_id (self->installation, xref);
-	if (g_strcmp0 (id, gs_app_get_unique_id (app)) == 0)
+	if (g_strcmp0 (gs_app_get_unique_id (app),
+	    gs_app_get_unique_id (app_tmp)) == 0)
 		return TRUE;
 
 	/* do all the metadata items match? */
@@ -1155,22 +1423,35 @@ gs_flatpak_set_app_metadata (GsFlatpak *self,
 
 static gboolean
 gs_plugin_refine_item_metadata (GsFlatpak *self,
-			       GsApp *app,
-			       GCancellable *cancellable,
-			       GError **error)
+				GsApp *app,
+				GCancellable *cancellable,
+				GError **error)
 {
 	const gchar *str;
 	gsize len = 0;
 	g_autofree gchar *contents = NULL;
 	g_autofree gchar *installation_path_str = NULL;
 	g_autofree gchar *install_path = NULL;
+	g_autoptr(AsProfileTask) ptask = NULL;
 	g_autoptr(GBytes) data = NULL;
 	g_autoptr(GFile) installation_path = NULL;
 
+	/* profile */
+	ptask = as_profile_start (gs_plugin_get_profile (self->plugin),
+				  "flatpak::refine-metadata{%s}",
+				  gs_app_get_id (app));
+	g_assert (ptask != NULL);
+
 	/* not applicable */
+	if (gs_app_get_kind (app) == AS_APP_KIND_SOURCE)
+		return TRUE;
 	if (gs_app_get_flatpak_kind (app) != FLATPAK_REF_KIND_APP)
 		return TRUE;
 
+	/* already done */
+	if (gs_app_has_kudo (app, GS_APP_KUDO_SANDBOXED))
+		return TRUE;
+
 	/* this is quicker than doing network IO */
 	installation_path = flatpak_installation_get_path (self->installation);
 	installation_path_str = g_file_get_path (installation_path);
@@ -1238,7 +1519,9 @@ gs_plugin_refine_item_size (GsFlatpak *self,
 	g_autoptr(AsProfileTask) ptask = NULL;
 
 	/* not applicable */
-	if (gs_app_get_state(app) == AS_APP_STATE_AVAILABLE_LOCAL)
+	if (gs_app_get_state (app) == AS_APP_STATE_AVAILABLE_LOCAL)
+		return TRUE;
+	if (gs_app_get_kind (app) == AS_APP_KIND_SOURCE)
 		return TRUE;
 
 	/* already set */
@@ -1324,6 +1607,21 @@ gs_plugin_refine_item_size (GsFlatpak *self,
 	return TRUE;
 }
 
+static gboolean
+gs_flatpak_refine_appstream (GsFlatpak *self, GsApp *app, GError **error)
+{
+	AsApp *item;
+	const gchar *unique_id = gs_app_get_unique_id (app);
+	if (unique_id == NULL)
+		return TRUE;
+	item = as_store_get_app_by_unique_id (self->store,
+					      unique_id,
+					      AS_STORE_SEARCH_FLAG_USE_WILDCARDS);
+	if (item == NULL)
+		return TRUE;
+	return gs_appstream_refine_app (self->plugin, app, item, error);
+}
+
 gboolean
 gs_flatpak_refine_app (GsFlatpak *self,
 		       GsApp *app,
@@ -1344,6 +1642,10 @@ gs_flatpak_refine_app (GsFlatpak *self,
 				  gs_app_get_id (app));
 	g_assert (ptask != NULL);
 
+	/* always do AppStream properties */
+	if (!gs_flatpak_refine_appstream (self, app, error))
+		return FALSE;
+
 	/* flatpak apps can always be removed */
 	gs_app_remove_quirk (app, AS_APP_QUIRK_COMPULSORY);
 
@@ -1411,6 +1713,48 @@ gs_flatpak_refine_app (GsFlatpak *self,
 	return TRUE;
 }
 
+gboolean
+gs_flatpak_refine_wildcard (GsFlatpak *self, GsApp *app,
+			    GsAppList *list, GsPluginRefineFlags flags,
+			    GCancellable *cancellable, GError **error)
+{
+	const gchar *id;
+	guint i;
+	g_autoptr(GPtrArray) items = NULL;
+
+	/* not valid */
+	id = gs_app_get_id (app);
+	if (id == NULL)
+		return TRUE;
+
+	/* find all apps when matching any prefixes */
+	items = as_store_get_apps_by_id (self->store, id);
+	for (i = 0; i < items->len; i++) {
+		AsApp *item = NULL;
+		g_autoptr(GsApp) new = NULL;
+
+		/* does the app have an installation method */
+		item = g_ptr_array_index (items, i);
+		if (as_app_get_bundle_default (item) == NULL) {
+			g_debug ("not using %s for wildcard as no bundle",
+				 as_app_get_id (item));
+			continue;
+		}
+
+		/* new app */
+		g_debug ("found %s for wildcard %s",
+			 as_app_get_unique_id (item), id);
+		new = gs_appstream_create_app (self->plugin, item, NULL);
+		if (new == NULL)
+			return FALSE;
+		gs_app_set_scope (new, self->scope);
+		if (!gs_flatpak_refine_app (self, new, flags, cancellable, error))
+			return FALSE;
+		gs_app_list_add (list, new);
+	}
+	return TRUE;
+}
+
 gboolean
 gs_flatpak_launch (GsFlatpak *self,
 		   GsApp *app,
@@ -1552,21 +1896,25 @@ gs_flatpak_app_install (GsFlatpak *self,
 		runtime = gs_app_get_runtime (app);
 
 		/* the runtime could come from a different remote to the app */
-		if (!gs_refine_item_metadata (self, runtime, cancellable,
-					      error))
+		if (!gs_refine_item_metadata (self, runtime, cancellable, error)) {
+			gs_app_set_state_recover (app);
 			return FALSE;
-		if (!gs_plugin_refine_item_origin (self,
-						   runtime, cancellable, error))
+		}
+		if (!gs_plugin_refine_item_origin (self, runtime, cancellable, error)) {
+			gs_app_set_state_recover (app);
 			return FALSE;
-		if (!gs_plugin_refine_item_state (self, runtime,
-						  cancellable, error))
+		}
+		if (!gs_plugin_refine_item_state (self, runtime, cancellable, error)) {
+			gs_app_set_state_recover (app);
 			return FALSE;
+		}
 		if (gs_app_get_state (runtime) == AS_APP_STATE_UNKNOWN) {
 			g_set_error (error,
 				     GS_PLUGIN_ERROR,
 				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
 				     "Failed to find runtime %s",
 				     gs_app_get_source_default (runtime));
+			gs_app_set_state_recover (app);
 			return FALSE;
 		}
 
@@ -1585,6 +1933,7 @@ gs_flatpak_app_install (GsFlatpak *self,
 							     cancellable, error);
 			if (xref == NULL) {
 				gs_app_set_state_recover (runtime);
+				gs_app_set_state_recover (app);
 				return FALSE;
 			}
 			gs_app_set_state (runtime, AS_APP_STATE_INSTALLED);
@@ -1639,7 +1988,7 @@ gs_flatpak_update_app (GsFlatpak *self,
 	/* install */
 	gs_app_set_state (app, AS_APP_STATE_INSTALLING);
 	xref = flatpak_installation_update (self->installation,
-					    FLATPAK_UPDATE_FLAGS_NO_PULL,
+					    FLATPAK_UPDATE_FLAGS_NONE,
 					    gs_app_get_flatpak_kind (app),
 					    gs_app_get_flatpak_name (app),
 					    gs_app_get_flatpak_arch (app),
@@ -1668,7 +2017,6 @@ gs_flatpak_file_to_app_bundle (GsFlatpak *self,
 {
 	gint size;
 	g_autofree gchar *content_type = NULL;
-	g_autofree gchar *unique_id = NULL;
 	g_autoptr(GBytes) appstream_gz = NULL;
 	g_autoptr(GBytes) icon_data = NULL;
 	g_autoptr(GBytes) metadata = NULL;
@@ -1682,17 +2030,8 @@ gs_flatpak_file_to_app_bundle (GsFlatpak *self,
 		return FALSE;
 	}
 
-	/* create a virtual ID */
-	unique_id = gs_flatpak_build_unique_id (self->installation,
-						FLATPAK_REF (xref_bundle));
-	app = gs_plugin_cache_lookup (self->plugin, unique_id);
-	if (app == NULL) {
-		g_autofree gchar *id = gs_flatpak_build_id (FLATPAK_REF (xref_bundle));
-		app = gs_app_new (id);
-		gs_plugin_cache_add (self->plugin, unique_id, app);
-	}
-
 	/* load metadata */
+	app = gs_plugin_create_app (self, FLATPAK_REF (xref_bundle));
 	gs_app_set_kind (app, AS_APP_KIND_DESKTOP);
 	gs_app_set_state (app, AS_APP_STATE_AVAILABLE_LOCAL);
 	gs_app_set_size_installed (app, flatpak_bundle_ref_get_installed_size (xref_bundle));
@@ -1768,6 +2107,12 @@ gs_flatpak_file_to_app_bundle (GsFlatpak *self,
 		/* copy details from AppStream to app */
 		if (!gs_appstream_refine_app (self->plugin, app, item, error))
 			return FALSE;
+	} else {
+		g_warning ("no appstream metadata in file");
+		gs_app_set_name (app, GS_APP_QUALITY_LOWEST,
+				 gs_app_get_flatpak_name (app));
+		gs_app_set_summary (app, GS_APP_QUALITY_LOWEST,
+				    "A flatpak application");
 	}
 
 	/* load icon */
@@ -1811,6 +2156,7 @@ gs_flatpak_file_to_app_repo (GsFlatpak *self,
 	g_autofree gchar *filename = NULL;
 	g_autofree gchar *repo_comment = NULL;
 	g_autofree gchar *repo_description = NULL;
+	g_autofree gchar *repo_default_branch = NULL;
 	g_autofree gchar *repo_gpgkey = NULL;
 	g_autofree gchar *repo_homepage = NULL;
 	g_autofree gchar *repo_icon = NULL;
@@ -1872,6 +2218,7 @@ gs_flatpak_file_to_app_repo (GsFlatpak *self,
 	gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
 	gs_app_set_name (app, GS_APP_QUALITY_NORMAL, repo_title);
 	gs_app_set_metadata (app, "flatpak::gpg-key", repo_gpgkey);
+	gs_app_set_metadata (app, "flatpak::url", repo_url);
 	gs_app_set_origin_hostname (app, repo_url);
 	gs_app_set_management_plugin (app, gs_plugin_get_name (self->plugin));
 
@@ -1885,6 +2232,9 @@ gs_flatpak_file_to_app_repo (GsFlatpak *self,
 	repo_description = g_key_file_get_string (kf, "Flatpak Repo", "Description", NULL);
 	if (repo_description != NULL)
 		gs_app_set_description (app, GS_APP_QUALITY_NORMAL, repo_description);
+	repo_default_branch = g_key_file_get_string (kf, "Flatpak Repo", "DefaultBranch", NULL);
+	if (repo_default_branch != NULL)
+		gs_app_set_branch (app, repo_default_branch);
 	repo_icon = g_key_file_get_string (kf, "Flatpak Repo", "Icon", NULL);
 	if (repo_icon != NULL) {
 		g_autoptr(AsIcon) ic = as_icon_new ();
@@ -1909,6 +2259,98 @@ gs_flatpak_file_to_app_repo (GsFlatpak *self,
 	return TRUE;
 }
 
+static gboolean
+gs_flatpak_file_to_app_ref (GsFlatpak *self,
+			    GsAppList *list,
+			    GFile *file,
+			    GCancellable *cancellable,
+			    GError **error)
+{
+	const gchar *remote_name;
+	gsize len = 0;
+	g_autofree gchar *contents = NULL;
+	g_autoptr(FlatpakRemoteRef) xref = NULL;
+	g_autoptr(GBytes) ref_file_data = NULL;
+	g_autoptr(GsApp) app = NULL;
+	g_autoptr(FlatpakRemote) xremote = NULL;
+	g_autofree gchar *origin_url = NULL;
+	g_autofree gchar *origin_title = NULL;
+
+	/* get file data */
+	if (!g_file_load_contents (file,
+				   cancellable,
+				   &contents,
+				   &len,
+				   NULL,
+				   error)) {
+		return FALSE;
+	}
+
+#if _FLATPAK_CHECK_VERSION(0,6,10)
+	/* install the remote, but not the app */
+	ref_file_data = g_bytes_new (contents, len);
+	xref = flatpak_installation_install_ref_file (self->installation,
+						      ref_file_data,
+						      cancellable,
+						      error);
+	if (xref == NULL) {
+		return FALSE;
+	}
+#else
+	g_set_error_literal (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+			     "not new enough libflatpak to support flatpakref");
+	return FALSE;
+#endif
+
+	/* load metadata */
+	app = gs_plugin_create_app (self, FLATPAK_REF (xref));
+	gs_app_set_kind (app, AS_APP_KIND_DESKTOP);
+	gs_app_set_state (app, AS_APP_STATE_AVAILABLE_LOCAL);
+	gs_flatpak_set_metadata (self, app, FLATPAK_REF (xref));
+
+	/* FIXME: perhaps use the data from the flatpakref file as a fallback? */
+
+	/* set the origin data */
+	remote_name = flatpak_remote_ref_get_remote_name (xref);
+	g_debug ("auto-created remote name: %s", remote_name);
+	xremote = flatpak_installation_get_remote_by_name (self->installation,
+							   remote_name,
+							   cancellable,
+							   error);
+	if (xremote == NULL) {
+		return FALSE;
+	}
+	origin_title = flatpak_remote_get_title (xremote);
+	origin_url = flatpak_remote_get_url (xremote);
+	if (origin_url == NULL) {
+		g_set_error (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_INVALID_FORMAT,
+			     "no URL for remote %s",
+			     flatpak_remote_get_name (xremote));
+		return FALSE;
+	}
+	gs_app_set_origin (app, remote_name);
+	gs_app_set_origin_hostname (app, origin_url);
+	gs_app_set_origin_ui (app, origin_title);
+
+	/* get the new appstream data */
+	if (!gs_flatpak_refresh_appstream_remote (self, remote_name,
+						  cancellable, error)) {
+		return FALSE;
+	}
+
+	/* parse it */
+	if (!gs_flatpak_add_apps_from_xremote (self, xremote, cancellable, error))
+		return FALSE;
+
+	/* success */
+	gs_app_list_add (list, app);
+	return TRUE;
+}
+
 gboolean
 gs_flatpak_file_to_app (GsFlatpak *self,
 			GsAppList *list,
@@ -1923,6 +2365,9 @@ gs_flatpak_file_to_app (GsFlatpak *self,
 	const gchar *mimetypes_repo[] = {
 		"application/vnd.flatpak.repo",
 		NULL };
+	const gchar *mimetypes_ref[] = {
+		"application/vnd.flatpak.ref",
+		NULL };
 
 	/* does this match any of the mimetypes_bundle we support */
 	content_type = gs_utils_get_content_type (file, cancellable, error);
@@ -1942,9 +2387,87 @@ gs_flatpak_file_to_app (GsFlatpak *self,
 						    cancellable,
 						    error);
 	}
+	if (g_strv_contains (mimetypes_ref, content_type)) {
+		return gs_flatpak_file_to_app_ref (self,
+						   list,
+						   file,
+						   cancellable,
+						   error);
+	}
 	return TRUE;
 }
 
+gboolean
+gs_flatpak_search (GsFlatpak *self,
+		   gchar **values,
+		   GsAppList *list,
+		   GCancellable *cancellable,
+		   GError **error)
+{
+	g_autoptr(GError) error_md = NULL;
+
+	/* if we've never ever run before, get the AppStream data */
+	if (!gs_flatpak_refresh_appstream (self, G_MAXUINT, 0,
+					   cancellable,
+					   &error_md)) {
+		g_warning ("failed to get initial available data: %s",
+			   error_md->message);
+	}
+
+	return gs_appstream_store_search (self->plugin, self->store,
+					  values, list,
+					  cancellable, error);
+}
+
+gboolean
+gs_flatpak_add_category_apps (GsFlatpak *self,
+			      GsCategory *category,
+			      GsAppList *list,
+			      GCancellable *cancellable,
+			      GError **error)
+{
+	return gs_appstream_store_add_category_apps (self->plugin, self->store,
+						     category, list,
+						     cancellable, error);
+}
+
+gboolean
+gs_flatpak_add_categories (GsFlatpak *self,
+			   GPtrArray *list,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	return gs_appstream_store_add_categories (self->plugin, self->store,
+						  list, cancellable, error);
+}
+
+gboolean
+gs_flatpak_add_popular (GsFlatpak *self,
+			GsAppList *list,
+			GCancellable *cancellable,
+			GError **error)
+{
+	return gs_appstream_add_popular (self->plugin, self->store, list,
+					 cancellable, error);
+}
+
+gboolean
+gs_flatpak_add_featured (GsFlatpak *self,
+			 GsAppList *list,
+			 GCancellable *cancellable,
+			 GError **error)
+{
+	return gs_appstream_add_featured (self->plugin, self->store, list,
+					  cancellable, error);
+}
+
+static void
+gs_flatpak_store_app_removed_cb (AsStore *store, AsApp *app, GsFlatpak *self)
+{
+	g_debug ("AppStream app was removed, doing delete from global cache");
+	gs_plugin_cache_remove (self->plugin, as_app_get_unique_id (app));
+}
+
 static void
 gs_flatpak_finalize (GObject *object)
 {
@@ -1953,6 +2476,7 @@ gs_flatpak_finalize (GObject *object)
 	self = GS_FLATPAK (object);
 
 	g_object_unref (self->plugin);
+	g_object_unref (self->store);
 	g_hash_table_unref (self->broken_remotes);
 
 	G_OBJECT_CLASS (gs_flatpak_parent_class)->finalize (object);
@@ -1970,6 +2494,12 @@ gs_flatpak_init (GsFlatpak *self)
 {
 	self->broken_remotes = g_hash_table_new_full (g_str_hash, g_str_equal,
 						      g_free, NULL);
+	self->store = as_store_new ();
+	g_signal_connect (self->store, "app-removed",
+			  G_CALLBACK (gs_flatpak_store_app_removed_cb),
+			  self);
+	as_store_set_add_flags (self->store, AS_STORE_ADD_FLAG_USE_UNIQUE_ID);
+	as_store_set_watch_flags (self->store, AS_STORE_WATCH_FLAG_REMOVED);
 }
 
 GsFlatpak *
