@@ -82,7 +82,7 @@ gs_plugin_detect_reload_apps (GsPlugin *plugin)
 		if (item == NULL) {
 			item = g_hash_table_lookup (app_hash, key);
 			app = gs_plugin_cache_lookup (plugin,
-						      as_app_get_id (item));
+						      as_app_get_unique_id (item));
 			if (app != NULL)
 				g_debug ("added GsApp %s", gs_app_get_id (app));
 			cnt++;
@@ -97,7 +97,7 @@ gs_plugin_detect_reload_apps (GsPlugin *plugin)
 		if (item == NULL) {
 			item = g_hash_table_lookup (priv->app_hash_old, key);
 			app = gs_plugin_cache_lookup (plugin,
-						      as_app_get_id (item));
+						      as_app_get_unique_id (item));
 			if (app != NULL)
 				g_debug ("removed GsApp %s", gs_app_get_id (app));
 			cnt++;
@@ -132,6 +132,15 @@ gs_plugin_appstream_store_changed_cb (AsStore *store, GsPlugin *plugin)
 		gs_plugin_reload (plugin);
 }
 
+static void
+gs_plugin_appstream_store_app_removed_cb (AsStore *store,
+					  AsApp *app,
+					  GsPlugin *plugin)
+{
+	g_debug ("AppStream app was removed, doing delete from global cache");
+	gs_plugin_cache_remove (plugin, as_app_get_unique_id (app));
+}
+
 void
 gs_plugin_initialize (GsPlugin *plugin)
 {
@@ -139,10 +148,18 @@ gs_plugin_initialize (GsPlugin *plugin)
 	priv->store = as_store_new ();
 	as_store_set_add_flags (priv->store,
 				AS_STORE_ADD_FLAG_USE_UNIQUE_ID |
+				AS_STORE_ADD_FLAG_ONLY_NATIVE_LANGS |
 				AS_STORE_ADD_FLAG_USE_MERGE_HEURISTIC);
 	as_store_set_watch_flags (priv->store,
 				  AS_STORE_WATCH_FLAG_ADDED |
 				  AS_STORE_WATCH_FLAG_REMOVED);
+	as_store_set_search_match (priv->store,
+				   AS_APP_SEARCH_MATCH_MIMETYPE |
+				   AS_APP_SEARCH_MATCH_PKGNAME |
+				   AS_APP_SEARCH_MATCH_COMMENT |
+				   AS_APP_SEARCH_MATCH_NAME |
+				   AS_APP_SEARCH_MATCH_KEYWORD |
+				   AS_APP_SEARCH_MATCH_ID);
 
 	/* set plugin flags */
 	gs_plugin_add_flags (plugin, GS_PLUGIN_FLAGS_GLOBAL_CACHE);
@@ -264,6 +281,12 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	g_signal_connect (priv->store, "changed",
 			  G_CALLBACK (gs_plugin_appstream_store_changed_cb),
 			  plugin);
+	g_signal_connect (priv->store, "app-removed",
+			  G_CALLBACK (gs_plugin_appstream_store_app_removed_cb),
+			  plugin);
+
+	/* ensure the token cache */
+	as_store_load_search_cache (priv->store);
 
 	/* add search terms for apps not in the main source */
 	origins = gs_plugin_appstream_get_origins_hash (items);
@@ -280,20 +303,6 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 		}
 	}
 
-	/* add keyword for non-package sources */
-	for (i = 0; i < items->len; i++) {
-		AsBundle *bundle;
-		app = g_ptr_array_index (items, i);
-		bundle = as_app_get_bundle_default (app);
-		if (bundle == NULL)
-			continue;
-		g_debug ("Adding keyword '%s' to %s",
-			 as_bundle_kind_to_string (as_bundle_get_kind (bundle)),
-			 as_app_get_unique_id (app));
-		as_app_add_keyword (app, NULL,
-				    as_bundle_kind_to_string (as_bundle_get_kind (bundle)));
-	}
-
 	/* fix up these */
 	for (i = 0; i < items->len; i++) {
 		app = g_ptr_array_index (items, i);
@@ -352,7 +361,7 @@ gs_plugin_refine_from_id (GsPlugin *plugin,
 	if (item == NULL) {
 		guint i;
 		GPtrArray *apps;
-		g_debug ("no app with ID %s found in appstream", unique_id);
+		g_debug ("no app with ID %s found in system appstream", unique_id);
 		apps = as_store_get_apps (priv->store);
 		for (i = 0; i < apps->len; i++) {
 			item = g_ptr_array_index (apps, i);
@@ -401,20 +410,6 @@ gs_plugin_refine_from_pkgname (GsPlugin *plugin,
 	return gs_appstream_refine_app (plugin, app, item, error);
 }
 
-static GsApp *
-gs_plugin_appstream_create_app (GsPlugin *plugin, AsApp *item)
-{
-	const gchar *unique_id = as_app_get_unique_id (item);
-	GsApp *app = gs_plugin_cache_lookup (plugin, unique_id);
-	if (app == NULL) {
-		app = gs_app_new (as_app_get_id (item));
-		gs_app_set_metadata (app, "GnomeSoftware::Creator",
-				     gs_plugin_get_name (plugin));
-		gs_plugin_cache_add (plugin, unique_id, app);
-	}
-	return app;
-}
-
 gboolean
 gs_plugin_add_distro_upgrades (GsPlugin *plugin,
 			       GsAppList *list,
@@ -435,11 +430,11 @@ gs_plugin_add_distro_upgrades (GsPlugin *plugin,
 			continue;
 
 		/* create */
-		app = gs_plugin_appstream_create_app (plugin, item);
+		app = gs_appstream_create_app (plugin, item, error);
+		if (app == NULL)
+			return FALSE;
 		gs_app_set_kind (app, AS_APP_KIND_OS_UPGRADE);
 		gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
-		if (!gs_appstream_refine_app (plugin, app, item, error))
-			return FALSE;
 		gs_app_list_add (list, app);
 	}
 	return TRUE;
@@ -454,10 +449,6 @@ gs_plugin_refine_app (GsPlugin *plugin,
 {
 	gboolean found = FALSE;
 
-	/* handled already */
-	if (gs_app_has_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX))
-		return TRUE;
-
 	/* find by ID then package name */
 	if (!gs_plugin_refine_from_id (plugin, app, &found, error))
 		return FALSE;
@@ -470,17 +461,20 @@ gs_plugin_refine_app (GsPlugin *plugin,
 	return TRUE;
 }
 
-static gboolean
-gs_plugin_appstream_add_wildcards (GsPlugin *plugin,
-				   GsAppList *list,
-				   GsApp *app,
-				   GError **error)
+gboolean
+gs_plugin_refine_wildcard (GsPlugin *plugin,
+			   GsApp *app,
+			   GsAppList *list,
+			   GsPluginRefineFlags flags,
+			   GCancellable *cancellable,
+			   GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	const gchar *id;
 	guint i;
 	g_autoptr(GPtrArray) items = NULL;
 
+	/* not enough info to find */
 	id = gs_app_get_id (app);
 	if (id == NULL)
 		return TRUE;
@@ -504,56 +498,16 @@ gs_plugin_appstream_add_wildcards (GsPlugin *plugin,
 		/* new app */
 		g_debug ("found %s for wildcard %s",
 			 as_app_get_id (item), id);
-		new = gs_plugin_appstream_create_app (plugin, item);
-		if (!gs_appstream_refine_app (plugin, new, item, error))
+		new = gs_appstream_create_app (plugin, item, error);
+		if (new == NULL)
 			return FALSE;
 		gs_app_list_add (list, new);
 	}
+
+	/* sucess */
 	return TRUE;
 }
 
-/* wildcard results get added to the list, not replaced */
-gboolean
-gs_plugin_refine (GsPlugin *plugin,
-		  GsAppList *list,
-		  GsPluginRefineFlags flags,
-		  GCancellable *cancellable,
-		  GError **error)
-{
-	guint i;
-
-	for (i = 0; i < gs_app_list_length (list); i++) {
-		GsApp *app = gs_app_list_index (list, i);
-		if (!gs_app_has_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX))
-			continue;
-		if (!gs_plugin_appstream_add_wildcards (plugin,
-							list,
-							app,
-							error))
-			return FALSE;
-	}
-
-	return TRUE;
-}
-
-static gboolean
-_as_app_matches_desktop_group_set (AsApp *app, gchar **desktop_groups)
-{
-	guint i;
-	for (i = 0; desktop_groups[i] != NULL; i++) {
-		if (!as_app_has_category (app, desktop_groups[i]))
-			return FALSE;
-	}
-	return TRUE;
-}
-
-static gboolean
-_as_app_matches_desktop_group (AsApp *app, const gchar *desktop_group)
-{
-	g_auto(GStrv) split = g_strsplit (desktop_group, "::", -1);
-	return _as_app_matches_desktop_group_set (app, split);
-}
-
 gboolean
 gs_plugin_add_category_apps (GsPlugin *plugin,
 			     GsCategory *category,
@@ -562,83 +516,12 @@ gs_plugin_add_category_apps (GsPlugin *plugin,
 			     GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	GPtrArray *array;
-	GPtrArray *desktop_groups;
-	guint i;
-	guint j;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* just look at each app in turn */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-category-apps");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (priv->store);
-	desktop_groups = gs_category_get_desktop_groups (category);
-	if (desktop_groups->len == 0) {
-		g_warning ("no desktop_groups for %s", gs_category_get_id (category));
-		return TRUE;
-	}
-	for (j = 0; j < desktop_groups->len; j++) {
-		const gchar *desktop_group = g_ptr_array_index (desktop_groups, j);
-		g_auto(GStrv) split = g_strsplit (desktop_group, "::", -1);
-
-		/* match the app */
-		for (i = 0; i < array->len; i++) {
-			AsApp *item;
-			g_autoptr(GsApp) app = NULL;
-
-			/* no ID is invalid */
-			item = g_ptr_array_index (array, i);
-			if (as_app_get_id (item) == NULL)
-				continue;
-
-			/* match all the desktop groups */
-			if (!_as_app_matches_desktop_group_set (item, split))
-				continue;
-
-			/* add all the data we can */
-			app = gs_plugin_appstream_create_app (plugin, item);
-			if (!gs_appstream_refine_app (plugin, app, item, error))
-				return FALSE;
-			gs_app_list_add (list, app);
-		}
-	}
-	return TRUE;
-}
-
-static gboolean
-gs_plugin_add_search_item (GsPlugin *plugin,
-			   GsAppList *list,
-			   AsApp *item,
-			   gchar **values,
-			   GCancellable *cancellable,
-			   GError **error)
-{
-	AsApp *item_tmp;
-	GPtrArray *addons;
-	guint i;
-	guint match_value;
-	g_autoptr(GsApp) app = NULL;
-
-	/* match against the app or any of the addons */
-	match_value = as_app_search_matches_all (item, values);
-	addons = as_app_get_addons (item);
-	for (i = 0; i < addons->len; i++) {
-		item_tmp = g_ptr_array_index (addons, i);
-		match_value |= as_app_search_matches_all (item_tmp, values);
-	}
-
-	/* no match */
-	if (match_value == 0)
-		return TRUE;
-
-	/* create app */
-	app = gs_plugin_appstream_create_app (plugin, item);
-	if (!gs_appstream_refine_app (plugin, app, item, error))
-		return FALSE;
-	gs_app_set_match_value (app, match_value);
-	gs_app_list_add (list, app);
-	return TRUE;
+	return gs_appstream_store_add_category_apps (plugin,
+						     priv->store,
+						     category,
+						     list,
+						     cancellable,
+						     error);
 }
 
 gboolean
@@ -649,27 +532,12 @@ gs_plugin_add_search (GsPlugin *plugin,
 		      GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	AsApp *item;
-	GPtrArray *array;
-	gboolean ret = TRUE;
-	guint i;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* search categories for the search term */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::search");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (priv->store);
-	for (i = 0; i < array->len; i++) {
-		if (g_cancellable_set_error_if_cancelled (cancellable, error))
-			return FALSE;
-
-		item = g_ptr_array_index (array, i);
-		ret = gs_plugin_add_search_item (plugin, list, item, values, cancellable, error);
-		if (!ret)
-			return FALSE;
-	}
-	return TRUE;
+	return gs_appstream_store_search (plugin,
+					  priv->store,
+					  values,
+					  list,
+					  cancellable,
+					  error);
 }
 
 gboolean
@@ -693,8 +561,8 @@ gs_plugin_add_installed (GsPlugin *plugin,
 		item = g_ptr_array_index (array, i);
 		if (as_app_get_state (item) == AS_APP_STATE_INSTALLED) {
 			g_autoptr(GsApp) app = NULL;
-			app = gs_plugin_appstream_create_app (plugin, item);
-			if (!gs_appstream_refine_app (plugin, app, item, error))
+			app = gs_appstream_create_app (plugin, item, error);
+			if (app == NULL)
 				return FALSE;
 			gs_app_list_add (list, app);
 		}
@@ -702,36 +570,6 @@ gs_plugin_add_installed (GsPlugin *plugin,
 	return TRUE;
 }
 
-static void
-gs_plugin_add_categories_for_app (GsCategory *parent, AsApp *app)
-{
-	GPtrArray *children;
-	GPtrArray *desktop_groups;
-	GsCategory *category;
-	guint i, j;
-
-	/* find all the sub-categories */
-	children = gs_category_get_children (parent);
-	for (j = 0; j < children->len; j++) {
-		gboolean matched = FALSE;
-		category = GS_CATEGORY (g_ptr_array_index (children, j));
-
-		/* do any desktop_groups match this application */
-		desktop_groups = gs_category_get_desktop_groups (category);
-		for (i = 0; i < desktop_groups->len; i++) {
-			const gchar *desktop_group = g_ptr_array_index (desktop_groups, i);
-			if (_as_app_matches_desktop_group (app, desktop_group)) {
-				matched = TRUE;
-				break;
-			}
-		}
-		if (matched) {
-			gs_category_increment_size (category);
-			gs_category_increment_size (parent);
-		}
-	}
-}
-
 gboolean
 gs_plugin_add_categories (GsPlugin *plugin,
 			  GPtrArray *list,
@@ -739,29 +577,8 @@ gs_plugin_add_categories (GsPlugin *plugin,
 			  GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	AsApp *app;
-	GPtrArray *array;
-	guint i;
-	guint j;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* find out how many packages are in each category */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-categories");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (priv->store);
-	for (i = 0; i < array->len; i++) {
-		app = g_ptr_array_index (array, i);
-		if (as_app_get_id (app) == NULL)
-			continue;
-		if (as_app_get_priority (app) < 0)
-			continue;
-		for (j = 0; j < list->len; j++) {
-			GsCategory *parent = GS_CATEGORY (g_ptr_array_index (list, j));
-			gs_plugin_add_categories_for_app (parent, app);
-		}
-	}
-	return TRUE;
+	return gs_appstream_store_add_categories (plugin, priv->store, list,
+						  cancellable, error);
 }
 
 gboolean
@@ -771,28 +588,8 @@ gs_plugin_add_popular (GsPlugin *plugin,
 		       GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	AsApp *item;
-	GPtrArray *array;
-	guint i;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* find out how many packages are in each category */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-popular");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (priv->store);
-	for (i = 0; i < array->len; i++) {
-		g_autoptr(GsApp) app = NULL;
-		item = g_ptr_array_index (array, i);
-		if (as_app_get_id (item) == NULL)
-			continue;
-		if (!as_app_has_kudo (item, "GnomeSoftware::popular"))
-			continue;
-		app = gs_plugin_appstream_create_app (plugin, item);
-		gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
-		gs_app_list_add (list, app);
-	}
-	return TRUE;
+	return gs_appstream_add_popular (plugin, priv->store, list, cancellable,
+					 error);
 }
 
 gboolean
@@ -802,28 +599,6 @@ gs_plugin_add_featured (GsPlugin *plugin,
 			GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	AsApp *item;
-	GPtrArray *array;
-	guint i;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* find out how many packages are in each category */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-featured");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (priv->store);
-	for (i = 0; i < array->len; i++) {
-		g_autoptr(GsApp) app = NULL;
-		item = g_ptr_array_index (array, i);
-		if (as_app_get_id (item) == NULL)
-			continue;
-		if (as_app_get_metadata_item (item, "GnomeSoftware::FeatureTile-css") == NULL)
-			continue;
-		app = gs_plugin_appstream_create_app (plugin, item);
-		if (!gs_appstream_refine_app (plugin, app, item, error))
-			return FALSE;
-		gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
-		gs_app_list_add (list, app);
-	}
-	return TRUE;
+	return gs_appstream_add_featured (plugin, priv->store, list, cancellable,
+					  error);
 }
