@@ -77,6 +77,24 @@ gboolean
 gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
+	const gchar *action_id = "org.freedesktop.Flatpak.appstream-update";
+	g_autoptr(GPermission) permission = NULL;
+
+	/* if we can't update the AppStream database system-wide don't even
+	 * pull the data as we can't do anything with it */
+	permission = gs_utils_get_permission (action_id);
+	if (permission != NULL) {
+		gboolean ret = g_permission_get_allowed (permission) ||
+				g_permission_get_can_acquire (permission);
+		if (!ret) {
+			g_set_error (error,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
+				     "no way to update using %s", action_id);
+			return FALSE;
+		}
+	}
+
 	return gs_flatpak_setup (priv->flatpak, cancellable, error);
 }
 
@@ -134,6 +152,19 @@ gs_plugin_refine_app (GsPlugin *plugin,
 				      cancellable, error);
 }
 
+gboolean
+gs_plugin_refine_wildcard (GsPlugin *plugin,
+			   GsApp *app,
+			   GsAppList *list,
+			   GsPluginRefineFlags flags,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	return gs_flatpak_refine_wildcard (priv->flatpak, app, list, flags,
+					   cancellable, error);
+}
+
 gboolean
 gs_plugin_launch (GsPlugin *plugin,
 		  GsApp *app,
@@ -193,3 +224,70 @@ gs_plugin_file_to_app (GsPlugin *plugin,
 	return gs_flatpak_file_to_app (priv->flatpak, list, file,
 				       cancellable, error);
 }
+
+gboolean
+gs_plugin_add_search (GsPlugin *plugin,
+		      gchar **values,
+		      GsAppList *list,
+		      GCancellable *cancellable,
+		      GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	return gs_flatpak_search (priv->flatpak,
+				  values,
+				  list,
+				  cancellable,
+				  error);
+}
+
+gboolean
+gs_plugin_add_categories (GsPlugin *plugin,
+			  GPtrArray *list,
+			  GCancellable *cancellable,
+			  GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	return gs_flatpak_add_categories (priv->flatpak, list,
+					  cancellable, error);
+}
+
+gboolean
+gs_plugin_add_category_apps (GsPlugin *plugin,
+			     GsCategory *category,
+			     GsAppList *list,
+			     GCancellable *cancellable,
+			     GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	return gs_flatpak_add_category_apps (priv->flatpak,
+					     category,
+					     list,
+					     cancellable,
+					     error);
+}
+
+gboolean
+gs_plugin_add_popular (GsPlugin *plugin,
+		       GsAppList *list,
+		       GCancellable *cancellable,
+		       GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	return gs_flatpak_add_popular (priv->flatpak,
+				       list,
+				       cancellable,
+				       error);
+}
+
+gboolean
+gs_plugin_add_featured (GsPlugin *plugin,
+			GsAppList *list,
+			GCancellable *cancellable,
+			GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	return gs_flatpak_add_featured (priv->flatpak,
+					list,
+					cancellable,
+					error);
+}
