@@ -361,6 +361,8 @@ gs_plugin_fwupd_new_app_from_results (GsPlugin *plugin, FwupdResult *res)
 	if (fwupd_result_get_update_uri (res) != NULL) {
 		gs_app_set_origin_hostname (app,
 					    fwupd_result_get_update_uri (res));
+		gs_app_set_metadata (app, "fwupd::UpdateURI",
+				     fwupd_result_get_update_uri (res));
 	}
 	if (fwupd_result_get_device_description (res) != NULL) {
 		g_autofree gchar *tmp = NULL;
@@ -400,7 +402,6 @@ gs_plugin_add_update_app (GsPlugin *plugin,
 			  FwupdResult *res,
 			  GError **error)
 {
-	FwupdDeviceFlags flags = 0;
 	const gchar *update_hash;
 	const gchar *update_uri;
 	g_autofree gchar *basename = NULL;
@@ -435,64 +436,58 @@ gs_plugin_add_update_app (GsPlugin *plugin,
 		return TRUE;
 	}
 
-	/* devices that are locked need unlocking */
-	flags = fwupd_result_get_device_flags (res);
-	if (flags & FU_DEVICE_FLAG_LOCKED) {
-		gs_app_set_metadata (app, "fwupd::IsLocked", "");
-	} else {
-		if (update_hash == NULL) {
-			g_set_error (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_NO_SECURITY,
-				     "%s [%s] (%s) has no checksum, ignoring as unsafe",
-				     gs_app_get_name (app),
-				     gs_app_get_id (app),
-				     gs_app_get_update_version (app));
-			return FALSE;
-		}
-		update_uri = fwupd_result_get_update_uri (res);
-		if (update_uri == NULL) {
-			g_set_error (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_INVALID_FORMAT,
-				     "no location available for %s [%s]",
-				     gs_app_get_name (app), gs_app_get_id (app));
-			return FALSE;
-		}
+	if (update_hash == NULL) {
+		g_set_error (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_NO_SECURITY,
+			     "%s [%s] (%s) has no checksum, ignoring as unsafe",
+			     gs_app_get_name (app),
+			     gs_app_get_id (app),
+			     gs_app_get_update_version (app));
+		return FALSE;
+	}
+	update_uri = fwupd_result_get_update_uri (res);
+	if (update_uri == NULL) {
+		g_set_error (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_INVALID_FORMAT,
+			     "no location available for %s [%s]",
+			     gs_app_get_name (app), gs_app_get_id (app));
+		return FALSE;
+	}
 
-		/* does the firmware already exist in the cache? */
-		basename = g_path_get_basename (update_uri);
-		filename_cache = gs_utils_get_cache_filename ("firmware",
-							      basename,
-							      GS_UTILS_CACHE_FLAG_NONE,
-							      error);
-		if (filename_cache == NULL)
-			return FALSE;
-		if (!g_file_test (filename_cache, G_FILE_TEST_EXISTS)) {
-			gs_plugin_fwupd_add_required_location (plugin, update_uri);
-			g_set_error (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_INVALID_FORMAT,
-				     "%s does not yet exist, wait patiently",
-				     filename_cache);
-			return FALSE;
-		}
+	/* does the firmware already exist in the cache? */
+	basename = g_path_get_basename (update_uri);
+	filename_cache = gs_utils_get_cache_filename ("firmware",
+						      basename,
+						      GS_UTILS_CACHE_FLAG_NONE,
+						      error);
+	if (filename_cache == NULL)
+		return FALSE;
+	if (!g_file_test (filename_cache, G_FILE_TEST_EXISTS)) {
+		gs_plugin_fwupd_add_required_location (plugin, update_uri);
+		g_set_error (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_INVALID_FORMAT,
+			     "%s does not yet exist, wait patiently",
+			     filename_cache);
+		return FALSE;
+	}
 
-		/* does the checksum match */
-		checksum = gs_plugin_fwupd_get_file_checksum (filename_cache,
-							      G_CHECKSUM_SHA1,
-							      error);
-		if (checksum == NULL)
-			return FALSE;
-		if (g_strcmp0 (update_hash, checksum) != 0) {
-			g_set_error (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_INVALID_FORMAT,
-				     "%s does not match checksum, expected %s got %s",
-				     filename_cache, update_hash, checksum);
-			g_unlink (filename_cache);
-			return FALSE;
-		}
+	/* does the checksum match */
+	checksum = gs_plugin_fwupd_get_file_checksum (filename_cache,
+						      G_CHECKSUM_SHA1,
+						      error);
+	if (checksum == NULL)
+		return FALSE;
+	if (g_strcmp0 (update_hash, checksum) != 0) {
+		g_set_error (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_INVALID_FORMAT,
+			     "%s does not match checksum, expected %s got %s",
+			     filename_cache, update_hash, checksum);
+		g_unlink (filename_cache);
+		return FALSE;
 	}
 
 	/* actually add the application */
@@ -571,6 +566,17 @@ gs_plugin_add_updates (GsPlugin *plugin,
 	for (i = 0; i < results->len; i++) {
 		FwupdResult *res = g_ptr_array_index (results, i);
 		g_autoptr(GError) error_local2 = NULL;
+
+		/* locked device that needs unlocking */
+		if (fwupd_result_get_device_flags (res) & FU_DEVICE_FLAG_LOCKED) {
+			g_autoptr(GsApp) app = NULL;
+			app = gs_plugin_fwupd_new_app_from_results (plugin, res);
+			gs_app_set_metadata (app, "fwupd::IsLocked", "");
+			gs_app_list_add (list, app);
+			continue;
+		}
+
+		/* normal device update */
 		if (!gs_plugin_add_update_app (plugin, list, res, &error_local2))
 			g_debug ("%s", error_local2->message);
 	}
@@ -697,11 +703,9 @@ gs_plugin_refresh (GsPlugin *plugin,
 
 	/* download the files to the cachedir */
 	for (i = 0; i < priv->to_download->len; i++) {
-		guint status_code;
 		g_autoptr(GError) error_local = NULL;
 		g_autofree gchar *basename = NULL;
 		g_autofree gchar *filename_cache = NULL;
-		g_autoptr(SoupMessage) msg = NULL;
 
 		tmp = g_ptr_array_index (priv->to_download, i);
 		basename = g_path_get_basename (tmp);
@@ -711,31 +715,20 @@ gs_plugin_refresh (GsPlugin *plugin,
 							      error);
 		if (filename_cache == NULL)
 			return FALSE;
-		g_debug ("downloading %s to %s", tmp, filename_cache);
 
-		/* set sync request */
-		msg = soup_message_new (SOUP_METHOD_GET, tmp);
-		status_code = soup_session_send_message (gs_plugin_get_soup_session (plugin), msg);
-		if (status_code != SOUP_STATUS_OK) {
+		/* download file */
+		if (!gs_plugin_download_file (plugin,
+					      NULL, /* app */
+					      tmp, /* url */
+					      filename_cache,
+					      cancellable,
+					      &error_local)) {
 			g_warning ("Failed to download %s, ignoring: %s",
-				   tmp, soup_status_get_phrase (status_code));
+				   tmp, error_local->message);
 			g_ptr_array_remove_index (priv->to_download, i--);
 			g_ptr_array_add (priv->to_ignore, g_strdup (tmp));
 			continue;
 		}
-
-		/* save binary file */
-		if (!g_file_set_contents (filename_cache,
-					  msg->response_body->data,
-					  msg->response_body->length,
-					  &error_local)) {
-			g_set_error (error,
-				     GS_PLUGIN_ERROR,
-				     GS_PLUGIN_ERROR_WRITE_FAILED,
-				     "Failed to save firmware: %s",
-				     error_local->message);
-			return FALSE;
-		}
 	}
 
 	return TRUE;
@@ -750,10 +743,12 @@ gs_plugin_fwupd_install (GsPlugin *plugin,
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	const gchar *device_id;
 	FwupdInstallFlags install_flags = 0;
+	GFile *local_file;
 	g_autofree gchar *filename = NULL;
 
 	/* not set */
-	if (gs_app_get_local_file (app) == NULL) {
+	local_file = gs_app_get_local_file (app);
+	if (local_file == NULL) {
 		g_set_error (error,
 			     GS_PLUGIN_ERROR,
 			     GS_PLUGIN_ERROR_FAILED,
@@ -761,7 +756,15 @@ gs_plugin_fwupd_install (GsPlugin *plugin,
 			     filename);
 		return FALSE;
 	}
-	filename = g_file_get_path (gs_app_get_local_file (app));
+
+	/* file does not yet exist */
+	filename = g_file_get_path (local_file);
+	if (!g_file_query_exists (local_file, cancellable)) {
+		const gchar *uri = gs_app_get_metadata_item (app, "fwupd::UpdateURI");
+		if (!gs_plugin_download_file (plugin, app, uri, filename,
+					      cancellable, error))
+			return FALSE;
+	}
 
 	/* limit to single device? */
 	device_id = gs_app_get_metadata_item (app, "fwupd::DeviceID");
